/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack;

import android.content.Context;
import android.graphics.Color;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.TextView;

import com.satsuware.usefulviews.LabelledSpinner;

import edu.uoc.sporttrack.adapter.OnItemActionListener;
import edu.uoc.sporttrack.adapter.RecyclerAdapter;
import edu.uoc.sporttrack.model.Itemizable;

public class BaseActivity extends AppCompatActivity implements OnItemActionListener
{
    /**
     * Renders the toolbar in activity
     */
    public Toolbar renderToolbar(boolean enableHome, String title)
    {
        Toolbar toolbar = findViewById(R.id.toolbar);
        toolbar.setTitle(title);
        setSupportActionBar(toolbar);
        if (enableHome && getSupportActionBar() != null)
        {
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        }

        return toolbar;
    }

    /**
     * Renders the toolbar in activity getting title from resource id
     */
    public Toolbar renderToolbar(boolean enableHome, int idTitle)
    {
        Toolbar toolbar = findViewById(R.id.toolbar);
        toolbar.setTitle(idTitle);
        setSupportActionBar(toolbar);
        if (enableHome && getSupportActionBar() != null)
        {
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        }

        return toolbar;
    }

    /**
     * Clears focus and hides soft keyboard
     */
    public void clearFocus(View... views)
    {
        for (View view : views)
        {
            if (view != null)
            {
                view.clearFocus();
            }
        }

        InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
        if (imm != null && views[0] != null)
        {
            imm.hideSoftInputFromWindow(views[0].getWindowToken(), 0);
        }
    }

    /**
     * Customizes labelled spinner
     */
    public void customizeLabelledSpinner(LabelledSpinner spinner)
    {
        // Removes error label from spinner
        spinner.removeView(spinner.getErrorLabel());

        // Sets label paddings
        TextView label = spinner.getLabel();
        label.setPadding(0, 0, 0, 0);

        // Sets divider margins
        View divider = spinner.getDivider();
        ViewGroup.MarginLayoutParams dividerParams = (ViewGroup.MarginLayoutParams) divider.getLayoutParams();
        dividerParams.bottomMargin = (int) getResources().getDimension(R.dimen.margin_small);
        divider.setLayoutParams(dividerParams);

        // Sets spinner paddings
        spinner.getSpinner()
               .setPadding(0, (int) getResources().getDimension(R.dimen.margin_medium), 0, (int) getResources().getDimension(R.dimen.margin_xsmall));
    }

    /**
     * Validates if form fields are informed
     */
    public boolean validateMandatoryFields(View... views)
    {
        for (View view : views)
        {
            if (view instanceof EditText)
            {
                if (((EditText) view).getText().toString().trim().isEmpty())
                {
                    // If no name is informed, an error is shown
                    view.requestFocus();
                    ((EditText) view).setError(getResources().getString(R.string.err_required_field));
                    return false;
                }
            }
            else if (view instanceof LabelledSpinner)
            {
                if (((LabelledSpinner) view).getSpinner().getSelectedItem().toString().trim().isEmpty())
                {
                    // If no item is selected, an error is shown
                    TextView textView = (TextView) ((LabelledSpinner) view).getSpinner().getSelectedView();
                    textView.setTextColor(Color.RED);
                    textView.setText(R.string.err_required_field);
                    return false;
                }
            }
        }

        return true;
    }

    /**
     * Callback method when a item in list view is clicked.
     * See {@link OnItemActionListener#onItemSelect(Itemizable)}
     */
    @Override
    public void onItemSelect(Itemizable itemizable)
    {
        // Does nothing
    }

    /**
     * Configures when a item is to be deleted.
     * See {@link OnItemActionListener#onItemDelete(Itemizable)}
     */
    @Override
    public void onItemDelete(final Itemizable itemizable)
    {
        // Does nothing
    }

    /**
     * Configures when a item is to be edited.
     * See {@link OnItemActionListener#onItemEdit(Itemizable)}
     */
    @Override
    public void onItemEdit(Itemizable itemizable)
    {
        // Does nothing
    }

    /**
     * Configures when a item is dragged.
     * See {@link OnItemActionListener#onItemDrag(RecyclerAdapter.ItemHolder)}
     */
    @Override
    public void onItemDrag(RecyclerAdapter.ItemHolder viewHolder)
    {
        // Does nothing
    }

    /**
     * Called when an item is dragged from one position to another.
     * See {@link OnItemActionListener#onItemMove(int, int)}
     */
    @Override
    public void onItemMove(int fromPosition, int toPosition)
    {
        // Does nothing
    }

    /**
     * Called when an item is dragged from one position to another.
     * See {@link OnItemActionListener#onItemMoved(RecyclerAdapter.ItemHolder)}
     */
    @Override
    public void onItemMoved(RecyclerAdapter.ItemHolder holder)
    {
        // Does nothing
    }
}
