/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack;

import android.content.Intent;
import android.os.Bundle;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.sporttrack.adapter.OnItemActionListener;
import edu.uoc.sporttrack.adapter.RecyclerAdapter;
import edu.uoc.sporttrack.model.Bodypart;
import edu.uoc.sporttrack.model.Day;
import edu.uoc.sporttrack.model.Itemizable;
import edu.uoc.sporttrack.model.Session;
import edu.uoc.sporttrack.utils.Utils;

public class BodypartActivity extends BaseDrawerActivity
{
    private int[] bodypartImages = new int[]{R.drawable.img_abs, R.drawable.img_back, R.drawable.img_biceps, R.drawable.img_chest,
                                             R.drawable.img_forearms, R.drawable.img_glutes, R.drawable.img_lowerlegs, R.drawable.img_shoulders,
                                             R.drawable.img_triceps, R.drawable.img_upperlegs};

    private Day day = null;
    private Session session = null;
    private boolean selectionMode = false;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        // Retrieves the content view that renders the activity.
        setContentView(R.layout.bodypart_activity);

        // Retrieves data from intent
        Bundle bundle = getIntent().getBundleExtra(Utils.BUNDLE);
        if (bundle != null)
        {
            selectionMode = bundle.getBoolean(Utils.ACCESS_MODE, false);
            day = bundle.getParcelable(Utils.DAY);
            session = bundle.getParcelable(Utils.SESSION);
        }

        // Renders the toolbar in activity
        Toolbar toolbar = renderToolbar(selectionMode, selectionMode ? R.string.exercise_title_select : R.string.exercise_title);

        if (!selectionMode)
        {
            // Access is from menu
            renderNavigationDrawer(toolbar);
        }

        displayBodyparts();
    }

    /**
     * Populates view list with items
     */
    private void displayBodyparts()
    {
        // Gets exercise array from resources
        String[] bodypartNames = getResources().getStringArray(R.array.bodyparts);

        // Creates adapter for recycler view
        List<Itemizable> items = new ArrayList<>();

        for (int i = 0; i < bodypartNames.length; i++)
        {
            Bodypart item = new Bodypart();
            item.setName(bodypartNames[i]);
            item.setImage(bodypartImages[i]);
            item.setId(i + 1);
            items.add(item);
        }

        RecyclerAdapter adapter = new RecyclerAdapter(this, R.layout.row_images, items, this, true);

        // Populates recycler view with items
        RecyclerView recyclerView = findViewById(R.id.bodypartsRecyclerView);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));
        recyclerView.setAdapter(adapter);
    }

    /**
     * Sets up the options menu in action bar
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        // Shows toolbar when not in selection mode
        if (!selectionMode)
        {
            // Inflates the menu, adding items to the action bar
            getMenuInflater().inflate(R.menu.toolbar_add, menu);
        }
        return true;
    }

    /**
     * Called whenever an item in action bar is selected.
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        switch (item.getItemId())
        {
            // If add icon is clicked
            case R.id.action_add:

                // Starts activity to add exercise and waits for response
                Intent intent = new Intent(getApplicationContext(), ExerciseEditActivity.class);
                startActivityForResult(intent, Utils.CODE_ADD_EXERCISE);
                return true;

            default:
                return super.onOptionsItemSelected(item);
        }
    }

    /**
     * Callback method to get result from other activity.
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data)
    {
        super.onActivityResult(requestCode, resultCode, data);

        // Checks if the request code is from the called activity and result is OK
        if (requestCode == Utils.CODE_ADD_EXERCISE && resultCode == Utils.RESULT_OK)
        {
            // Shows to the user that the action has been OK
            Toast.makeText(this, R.string.msg_exercise_saved, Toast.LENGTH_SHORT).show();
        }
    }

    /**
     * Callback method when a item in list view is clicked.
     * See {@link OnItemActionListener#onItemSelect(Itemizable)}
     */
    @Override
    public void onItemSelect(Itemizable item)
    {
        Bundle bundle = new Bundle();
        bundle.putParcelable(Utils.BODYPART, (Bodypart) item);
        bundle.putBoolean(Utils.ACCESS_MODE, selectionMode);
        bundle.putParcelable(Utils.DAY, day);
        bundle.putParcelable(Utils.SESSION, session);
        Intent intent = new Intent(getApplicationContext(), ExerciseActivity.class);
        intent.putExtra(Utils.BUNDLE, bundle);
        startActivity(intent);

    }
}
