/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack;

import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AlertDialog;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.helper.ItemTouchHelper;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.Toast;

import com.daimajia.swipe.util.Attributes;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import edu.uoc.sporttrack.adapter.DragHelper;
import edu.uoc.sporttrack.adapter.OnItemActionListener;
import edu.uoc.sporttrack.adapter.RecyclerAdapter;
import edu.uoc.sporttrack.database.PlanDatabaseHelper;
import edu.uoc.sporttrack.model.Day;
import edu.uoc.sporttrack.model.Itemizable;
import edu.uoc.sporttrack.model.Plan;
import edu.uoc.sporttrack.utils.Utils;

public class DayDetailActivity extends BaseActivity
{
    private PlanDatabaseHelper db;
    private RecyclerAdapter adapter;
    private List<Itemizable> items;
    private ItemTouchHelper helper;
    private Day day = null;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        // Retrieves the content view that renders the activity.
        setContentView(R.layout.day_detail_activity);

        // Retrieves data from intent
        Bundle bundle = getIntent().getBundleExtra(Utils.BUNDLE);
        if (bundle != null)
        {
            day = bundle.getParcelable(Utils.DAY);
            day.setContext(this);
        }

        // Renders the toolbar in activity
        renderToolbar(true, day != null ? day.getItemTitle() : getResources().getString(R.string.day_title_detail));

        // Creates database helper
        db = new PlanDatabaseHelper(this);

        // Lists all plans for selected day
        displayPlans();
    }

    /**
     * Lists all plans in recycler view for received day
     */
    private void displayPlans()
    {
        if (day != null)
        {
            // Gets items
            items = new ArrayList<>();
            items.addAll(db.getAll(day.getId()));

            // Creates adapter for recycler view
            adapter = new RecyclerAdapter(this, R.layout.row_plan, items, this, false);
            // Setting mode to single to reveal buttoms for one item
            adapter.setMode(Attributes.Mode.Single);

            // Creates recycler view and binds adapter
            RecyclerView recyclerView = findViewById(R.id.plansRecyclerView);
            recyclerView.setLayoutManager(new LinearLayoutManager(this));
            recyclerView.setAdapter(adapter);

            // Creates helper to drag items in list
            DragHelper callback = new DragHelper(ItemTouchHelper.UP | ItemTouchHelper.DOWN, 0, this);
            helper = new ItemTouchHelper(callback);
            helper.attachToRecyclerView(recyclerView);
        }
    }

    /**
     * Sets up the action bar.
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        // Inflates the menu, adding items to the action bar
        getMenuInflater().inflate(R.menu.toolbar_add, menu);
        return true;
    }

    /**
     * Called whenever an item in action bar is selected.
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        switch (item.getItemId())
        {
            // If add icon is clicked
            case R.id.action_add:

                // Starts activity and waits for response
                Bundle bundle = new Bundle();
                bundle.putBoolean(Utils.ACCESS_MODE, Utils.MODE_SELECTION);
                bundle.putParcelable(Utils.DAY, day);
                Intent intent = new Intent(getApplicationContext(), BodypartActivity.class);
                intent.putExtra(Utils.BUNDLE, bundle);
                startActivityForResult(intent, Utils.CODE_SELECT_EXERCISE);
                return true;

            default:
                return super.onOptionsItemSelected(item);
        }
    }

    /**
     * Callback method to get result from other activity.
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data)
    {
        super.onActivityResult(requestCode, resultCode, data);

        switch (requestCode)
        {
            case Utils.CODE_SELECT_EXERCISE:

                if (day != null)
                {
                    //Filters items in recycler view
                    refreshItems(null);
                }
                break;

            case Utils.CODE_UPDATE_PLAN:

                if (resultCode == Utils.RESULT_OK)
                {
                    //Filters items in recycler view
                    refreshItems(null);
                    Toast.makeText(this, R.string.msg_plan_updated, Toast.LENGTH_SHORT).show();
                }
                break;
        }
    }

    /**
     * Configures when a item is to be deleted.
     * See {@link OnItemActionListener#onItemDelete(Itemizable)}
     */
    @Override
    public void onItemDelete(final Itemizable itemizable)
    {
        // Shows dialog to confirm
        AlertDialog.Builder builder = new AlertDialog.Builder(DayDetailActivity.this);
        builder.setTitle(getResources().getString(R.string.plan_title_delete));
        builder.setMessage(getResources().getString(R.string.msg_plan_delete));
        builder.setCancelable(false);
        builder.setPositiveButton(getResources().getString(R.string.action_yes), new DialogInterface.OnClickListener()
        {
            @Override
            public void onClick(DialogInterface dialog, int which)
            {
                if (db.delete(itemizable.getId()) > 0)
                {
                    //Refreshes items in recycler view
                    refreshItems(null);

                    Toast.makeText(getApplicationContext(), getResources().getString(R.string.msg_plan_deleted), Toast.LENGTH_SHORT).show();
                }
                else
                {
                    Toast.makeText(getApplicationContext(), getResources().getString(R.string.err_plan_deleted), Toast.LENGTH_SHORT).show();
                }
            }
        });

        builder.setNegativeButton(getResources().getString(R.string.action_no), null);
        builder.show();
    }

    /**
     * Configures when a item is to be edited.
     * See {@link OnItemActionListener#onItemEdit(Itemizable)}
     */
    @Override
    public void onItemEdit(Itemizable itemizable)
    {
        // Starts activity and waits for response
        Bundle bundle = new Bundle();
        bundle.putParcelable(Utils.PLAN, (Plan) itemizable);
        Intent intent = new Intent(getApplicationContext(), PlanEditActivity.class);
        intent.putExtra(Utils.BUNDLE, bundle);
        startActivityForResult(intent, Utils.CODE_UPDATE_PLAN);
    }

    /**
     * Configures when a item is dragged.
     * See {@link OnItemActionListener#onItemDrag(RecyclerAdapter.ItemHolder)}
     */
    @Override
    public void onItemDrag(RecyclerAdapter.ItemHolder holder)
    {
        helper.startDrag(holder);
    }

    /**
     * Called when an item is dragged from one position to another.
     * See {@link OnItemActionListener#onItemMove(int, int)}
     */
    @Override
    public void onItemMove(int fromPosition, int toPosition)
    {
        Collections.swap(items, fromPosition, toPosition);
        adapter.notifyItemMoved(fromPosition, toPosition);
    }

    /**
     * Called when an item is dragged from one position to another.
     * See {@link OnItemActionListener#onItemMoved(RecyclerAdapter.ItemHolder)}
     */
    @Override
    public void onItemMoved(RecyclerAdapter.ItemHolder holder)
    {
        List<Plan> plans = new ArrayList<>();
        int pos = 1;
        for (Itemizable item : items)
        {
            Plan plan = (Plan) item;
            plan.setPosition(pos);
            plans.add(plan);
            pos++;
        }

        db.update(plans);
        refreshItems(plans);
    }

    /**
     * Refreshes items in list and notifies adapter in recycler view.
     */
    private void refreshItems(List<? extends Itemizable> newItems)
    {
        if (newItems == null && day != null)
        {
            adapter.getManager().closeAllItems();
            newItems = db.getAll(day.getId());
        }
        if (newItems != null)
        {
            this.items.clear();
            this.items.addAll(newItems);
        }
        adapter.notifyDataSetChanged();
    }
}
