/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack;

import android.content.Intent;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.satsuware.usefulviews.LabelledSpinner;
import com.satsuware.usefulviews.LabelledSpinner.OnItemChosenListener;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.sporttrack.database.DayDatabaseHelper;
import edu.uoc.sporttrack.model.Day;
import edu.uoc.sporttrack.model.Routine;
import edu.uoc.sporttrack.utils.Utils;

public class DayEditActivity extends BaseActivity implements OnItemChosenListener
{
    private static final int MAX_DAYS = 30;

    private DayDatabaseHelper db;
    private Routine routine;
    private Day day;
    private LabelledSpinner daySpinner;
    private EditText nameEditText;
    private int currentDay;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        // Retrieves the content view that renders the activity.
        setContentView(R.layout.day_add_activity);

        // Retrieves data from intent
        Bundle bundle = getIntent().getBundleExtra(Utils.BUNDLE);
        if (bundle != null)
        {
            day = bundle.getParcelable(Utils.DAY);
            routine = bundle.getParcelable(Utils.ROUTINE);
        }

        // Renders the toolbar in activity
        renderToolbar(true, day != null ? R.string.day_title_edit : R.string.day_title_add);

        // Creates database helper
        db = new DayDatabaseHelper(this);

        setupInitialValues();
    }

    /**
     * Sets up the action bar.
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        // Inflates the menu, adding items to the action bar
        getMenuInflater().inflate(R.menu.toolbar_save, menu);
        return true;
    }

    /**
     * Called whenever an item in action bar is selected.
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        switch (item.getItemId())
        {
            // When clicking on saving a routine day
            case R.id.action_save:

                nameEditText = findViewById(R.id.dayName);
                daySpinner = findViewById(R.id.daySpinnerDay);

                // If validations are ok, day is saved
                if (saveDay())
                {
                    // Finishes activity and sends OK code to waiting activity
                    Intent intent = new Intent();
                    setResult(Utils.RESULT_OK, intent);
                    finish();
                }
                return true;

            default:
                return super.onOptionsItemSelected(item);
        }
    }

    // Called when an item in spinner is selected
    @Override
    public void onItemChosen(View labelledSpinner, AdapterView<?> parent, View itemView, int position, long id)
    {
        // The position in the spinner corresponds to day id
        currentDay = position;
        clearFocus(findViewById(R.id.dayName));
    }

    // Called when no item in spinner is selected
    @Override
    public void onNothingChosen(View labelledSpinner, AdapterView<?> adapterView)
    {
        // Does nothing
    }

    /**
     * Sets up initial values.
     */
    private void setupInitialValues()
    {
        List<String> days = new ArrayList<>();
        for (int i = 1; i <= MAX_DAYS; i++)
        {
            days.add(String.valueOf(i));
        }
        days.add(0, "");

        // Adds array to spinner
        daySpinner = findViewById(R.id.daySpinnerDay);
        daySpinner.setItemsArray(days);
        daySpinner.setOnItemChosenListener(this);
        customizeLabelledSpinner(daySpinner);

        if (day != null)
        {
            nameEditText = findViewById(R.id.dayName);
            nameEditText.setText(day.getName());
            daySpinner.getSpinner().setSelection(day.getPosition());
        }

        TextView routineTextView = findViewById(R.id.dayRoutineName);
        routineTextView.setText(routine != null ? routine.getName() : "");
    }

    /**
     * Validates and saves routine day in database and returns true if ok
     */
    private boolean saveDay()
    {
        if (validateMandatoryFields(daySpinner, nameEditText))
        {
            if (day == null)
            {
                day = new Day();
            }

            // Setting valid values to routine day object
            day.setIdRoutine(routine != null ? routine.getId() : -1);
            day.setName(nameEditText.getText().toString().trim());
            day.setPosition(currentDay != -1 ? currentDay : 0);

            if (db.saveOrUpdate(day) > 0)
            {
                return true;
            }

            Toast.makeText(this, R.string.err_day_saved, Toast.LENGTH_SHORT).show();
        }

        return false;
    }

}
