/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack;

import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AlertDialog;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.SearchView;
import android.support.v7.widget.SearchView.OnQueryTextListener;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.sporttrack.adapter.OnItemActionListener;
import edu.uoc.sporttrack.adapter.RecyclerAdapter;
import edu.uoc.sporttrack.database.ExerciseDatabaseHelper;
import edu.uoc.sporttrack.database.PlanDatabaseHelper;
import edu.uoc.sporttrack.database.SessionDatabaseHelper;
import edu.uoc.sporttrack.database.SettingsDatabaseHelper;
import edu.uoc.sporttrack.model.Bodypart;
import edu.uoc.sporttrack.model.Day;
import edu.uoc.sporttrack.model.Exercise;
import edu.uoc.sporttrack.model.Itemizable;
import edu.uoc.sporttrack.model.LogPlan;
import edu.uoc.sporttrack.model.Plan;
import edu.uoc.sporttrack.model.Session;
import edu.uoc.sporttrack.model.Settings;
import edu.uoc.sporttrack.utils.Utils;

public class ExerciseActivity extends BaseActivity implements OnQueryTextListener
{
    private ExerciseDatabaseHelper exerciseDB;
    private PlanDatabaseHelper planDB;
    private SessionDatabaseHelper sessionDB;
    private RecyclerAdapter adapter;
    private Settings settings;
    private List<Itemizable> items;
    private Bodypart bodypart = null;
    private Day day = null;
    private Session session = null;
    private boolean selectionMode = false;
    private String searchedText = "";
    private String[] bodypartNames;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        // Retrieves the content view that renders the activity.
        setContentView(R.layout.exercise_activity);

        // Retrieves data from intent
        Bundle bundle = getIntent().getBundleExtra(Utils.BUNDLE);
        if (bundle != null)
        {
            bodypart = bundle.getParcelable(Utils.BODYPART);
            selectionMode = bundle.getBoolean(Utils.ACCESS_MODE, false);
            day = bundle.getParcelable(Utils.DAY);
            session = bundle.getParcelable(Utils.SESSION);
        }

        // Renders the toolbar in activity
        renderToolbar(true, bodypart != null ? bodypart.getName() : getResources().getString(R.string.exercise_title_select));

        // Creates database helpers
        exerciseDB = new ExerciseDatabaseHelper(this, selectionMode);
        planDB = new PlanDatabaseHelper(this);
        sessionDB = new SessionDatabaseHelper(this);
        settings = new SettingsDatabaseHelper(this).get();

        // Lists all exercises for selected body part
        displayExercises();

        // Gets exercise array from resources
        bodypartNames = getResources().getStringArray(R.array.bodyparts);
    }

    /**
     * Lists all exercises in recycler view for received body part
     */
    private void displayExercises()
    {
        if (bodypart != null)
        {
            // Creates recycler view and binds adapter
            RecyclerView recyclerView = findViewById(R.id.exercisesRecyclerView);
            recyclerView.setLayoutManager(new LinearLayoutManager(this));

            // Creates adapter for recycler view
            items = new ArrayList<>();
            adapter = new RecyclerAdapter(this, R.layout.row_general, items, this, selectionMode);
            items.addAll(exerciseDB.getAll(bodypart.getId(), null));

            recyclerView.setAdapter(adapter);
        }
    }

    /**
     * Sets up the options menu in action bar
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        // Inflates the menu, adding items to the action bar
        getMenuInflater().inflate(selectionMode ? R.menu.toolbar_search : R.menu.toolbar_search_add, menu);

        // Sets up search view in action bar
        MenuItem searchMenuItem = menu.findItem(R.id.action_search);
        SearchView searchView = (SearchView) searchMenuItem.getActionView();
        searchView.setQueryHint(getResources().getString(R.string.action_search));
        searchView.setOnQueryTextListener(this);

        return true;
    }

    /**
     * Called whenever an item in action bar is selected.
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        switch (item.getItemId())
        {
            // If add icon is clicked
            case R.id.action_add:

                // Starts activity and waits for response
                Bundle bundle = new Bundle();
                bundle.putParcelable(Utils.BODYPART, bodypart);
                Intent intent = new Intent(getApplicationContext(), ExerciseEditActivity.class);
                intent.putExtra(Utils.BUNDLE, bundle);
                startActivityForResult(intent, Utils.CODE_ADD_EXERCISE);
                return true;

            default:
                return super.onOptionsItemSelected(item);
        }
    }

    /**
     * Callback method to get result from other activity.
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data)
    {
        super.onActivityResult(requestCode, resultCode, data);

        // Checks if the request code is from the called activity and result is OK
        if (resultCode == Utils.RESULT_OK)
        {
            switch (requestCode)
            {
                case Utils.CODE_ADD_EXERCISE:

                    // Shows to the user that the action has been OK
                    Toast.makeText(this, R.string.msg_exercise_saved, Toast.LENGTH_SHORT).show();
                    break;

                case Utils.CODE_UPDATE_EXERCISE:

                    // Shows to the user that the action has been OK
                    Toast.makeText(this, R.string.msg_exercise_updated, Toast.LENGTH_SHORT).show();
                    break;
            }

            // Filters items in recycler view
            refreshItems();
        }
    }

    /**
     * Called when the user submits the query.
     */
    @Override
    public boolean onQueryTextSubmit(String query)
    {
        return false;
    }

    /**
     * Called when the query text is changed by the user.
     */
    @Override
    public boolean onQueryTextChange(String text)
    {
        //Filters items in recycler view
        if (!searchedText.equals(text))
        {
            searchedText = text;
            refreshItems();
        }
        return true;
    }

    /**
     * Callback method when a item in list view is clicked.
     * See {@link OnItemActionListener#onItemSelect(Itemizable)}
     */
    @Override
    public void onItemSelect(Itemizable itemizable)
    {
        // Different behaviour depending on access mode
        if (selectionMode)
        {
            if (day != null)
            {
                // Creates new plan to be saved
                Plan plan = new Plan();
                plan.setIdDay(day.getId());
                plan.setPosition(planDB.nextPosition(day.getId()));

                Exercise exercise = (Exercise) itemizable;
                plan.setIdExercise(exercise.getId());

                if (settings != null)
                {
                    plan.setSets(settings.getSets());
                    plan.setReps(settings.getReps());
                    plan.setRest(settings.getRest());
                }

                planDB.add(plan);

                if (session == null || session.getId() == 0)
                {
                    Toast.makeText(getApplicationContext(), getResources().getString(R.string.msg_exercise_added), Toast.LENGTH_SHORT).show();
                }
            }

            if (session != null && session.getId() != 0)
            {
                // Creates new log plan to be saved
                LogPlan log = new LogPlan();
                log.setIdSession(session.getId());

                Exercise exercise = (Exercise) itemizable;
                log.setIdExercise(exercise.getId());
                log.setExerciseName(exercise.getName());
                log.setBodypart(bodypartNames[(int) exercise.getIdBodypart() - 1]);

                // Retrieves log plans for session
                session.setLogPlans(sessionDB.getAllLogPlans(session.getId()));
                log.setPosition(session.getLogPlans().size() + 1);

                if (settings != null)
                {
                    log.setSets(settings.getSets());
                    log.setReps(settings.getReps());
                    log.setRest(settings.getRest());
                }

                sessionDB.add(log);

                Toast.makeText(getApplicationContext(), getResources().getString(R.string.msg_workout_added), Toast.LENGTH_SHORT).show();
            }
        }
        else
        {
            // Starts activity and waits for response
            Bundle bundle = new Bundle();
            bundle.putParcelable(Utils.EXERCISE, (Exercise) itemizable);
            bundle.putParcelable(Utils.BODYPART, bodypart);
            Intent intent = new Intent(getApplicationContext(), ExerciseDetailActivity.class);
            intent.putExtra(Utils.BUNDLE, bundle);
            startActivity(intent);
        }
    }

    /**
     * Configures when a item is to be deleted.
     * See {@link OnItemActionListener#onItemDelete(Itemizable)}
     */
    @Override
    public void onItemDelete(final Itemizable itemizable)
    {
        // Shows dialog to confirm
        AlertDialog.Builder builder = new AlertDialog.Builder(ExerciseActivity.this);
        builder.setTitle(getResources().getString(R.string.exercise_title_delete));
        builder.setMessage(getResources().getString(R.string.msg_exercise_delete));
        builder.setCancelable(false);
        builder.setPositiveButton(getResources().getString(R.string.action_yes), new DialogInterface.OnClickListener()
        {
            @Override
            public void onClick(DialogInterface dialog, int which)
            {
                if (exerciseDB.delete(itemizable.getId()) > 0)
                {
                    //Refreshes items in recycler view
                    refreshItems();

                    Toast.makeText(getApplicationContext(), getResources().getString(R.string.msg_exercise_deleted), Toast.LENGTH_SHORT).show();
                }
                else
                {
                    Toast.makeText(getApplicationContext(), getResources().getString(R.string.err_exercise_deleted), Toast.LENGTH_SHORT).show();
                }
            }
        });

        builder.setNegativeButton(getResources().getString(R.string.action_no), null);
        builder.show();
    }

    /**
     * Configures when a item is to be edited.
     * See {@link OnItemActionListener#onItemEdit(Itemizable)}
     */
    @Override
    public void onItemEdit(Itemizable itemizable)
    {
        // Starts activity and waits for response
        Bundle bundle = new Bundle();
        bundle.putParcelable(Utils.EXERCISE, (Exercise) itemizable);
        bundle.putParcelable(Utils.BODYPART, bodypart);
        Intent intent = new Intent(getApplicationContext(), ExerciseEditActivity.class);
        intent.putExtra(Utils.BUNDLE, bundle);
        startActivityForResult(intent, Utils.CODE_UPDATE_EXERCISE);
    }

    /**
     * Refreshes items in list and notifies adapter in recycler view.
     */
    private void refreshItems()
    {
        adapter.getManager().closeAllItems();
        this.items.clear();
        this.items.addAll(exerciseDB.getAll(bodypart.getId(), searchedText));
        adapter.notifyDataSetChanged();
    }
}
