/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package edu.uoc.sporttrack;

import android.content.Intent;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.EditText;
import android.widget.Toast;

import com.satsuware.usefulviews.LabelledSpinner;
import com.satsuware.usefulviews.LabelledSpinner.OnItemChosenListener;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import edu.uoc.sporttrack.database.ExerciseDatabaseHelper;
import edu.uoc.sporttrack.model.Bodypart;
import edu.uoc.sporttrack.model.Exercise;
import edu.uoc.sporttrack.utils.Utils;

public class ExerciseEditActivity extends BaseActivity implements OnItemChosenListener
{
    private ExerciseDatabaseHelper db;
    private LabelledSpinner bodypartSpinner;
    private EditText nameEditText, descEditText;
    private int currentBodypart;
    private Bodypart bodypart;
    private Exercise exercise;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        // Retrieves the content view that renders the activity.
        setContentView(R.layout.exercise_add_activity);

        // Retrieves data from intent
        Bundle bundle = getIntent().getBundleExtra(Utils.BUNDLE);
        if (bundle != null)
        {
            exercise = bundle.getParcelable(Utils.EXERCISE);
            bodypart = bundle.getParcelable(Utils.BODYPART);
        }

        // Creates database helper
        db = new ExerciseDatabaseHelper(this, false);

        // Renders the toolbar in activity
        renderToolbar(true, exercise != null ? R.string.exercise_title_edit : R.string.exercise_title_add);

        setupInitialValues();
    }

    // Called when an item in spinner is selected
    @Override
    public void onItemChosen(View labelledSpinner, AdapterView<?> parent, View itemView, int position, long id)
    {
        // The position in the spinner corresponds to group id
        currentBodypart = position;
        clearFocus(findViewById(R.id.exerciseName), findViewById(R.id.exerciseDesc));
    }

    // Called when no item in spinner is selected
    @Override
    public void onNothingChosen(View labelledSpinner, AdapterView<?> adapterView)
    {
        // does nothing
    }

    /**
     * Sets up the action bar.
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        // Inflates the menu, adding items to the action bar
        getMenuInflater().inflate(R.menu.toolbar_save, menu);
        return true;
    }

    /**
     * Called whenever an item in action bar is selected.
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        switch (item.getItemId())
        {
            // When clicking on saving an exercise
            case R.id.action_save:

                // Gets value from fields
                nameEditText = findViewById(R.id.exerciseName);
                bodypartSpinner = findViewById(R.id.exerciseBodypart);
                descEditText = findViewById(R.id.exerciseDesc);

                // If validations are ok, exercise is saved
                if (saveExercise())
                {
                    // Finishes activity and sends OK code to waiting activity
                    Intent intent = new Intent();
                    setResult(Utils.RESULT_OK, intent);
                    finish();
                }
                return true;

            default:
                return super.onOptionsItemSelected(item);
        }
    }

    /**
     * Sets up initial values.
     */
    private void setupInitialValues()
    {
        // Retrieves array and adds default blank value
        List<String> groups = new ArrayList<>(Arrays.asList(getResources().getStringArray(R.array.bodyparts)));
        groups.add(0, "");

        // Adds array to spinner
        bodypartSpinner = findViewById(R.id.exerciseBodypart);
        bodypartSpinner.setItemsArray(groups);
        bodypartSpinner.setOnItemChosenListener(this);
        customizeLabelledSpinner(bodypartSpinner);

        if (exercise != null)
        {
            nameEditText = findViewById(R.id.exerciseName);
            nameEditText.setText(exercise.getName());

            descEditText = findViewById(R.id.exerciseDesc);
            descEditText.setText(exercise.getDescription());
        }

        bodypartSpinner.getSpinner().setSelection(bodypart != null ? (int) bodypart.getId() : -1);
    }

    /**
     * Validates and saves exercise in database and returns true if ok
     */
    private boolean saveExercise()
    {
        if (validateMandatoryFields(nameEditText, bodypartSpinner))
        {
            if (exercise == null)
            {
                exercise = new Exercise();
            }

            exercise.setName(nameEditText.getText().toString().trim());
            exercise.setIdBodypart(currentBodypart);

            if (!descEditText.getText().toString().trim().isEmpty())
            {
                exercise.setDescription(descEditText.getText().toString().trim());
            }

            if (db.saveOrUpdate(exercise) > 0)
            {
                return true;
            }

            Toast.makeText(this, R.string.err_exercise_saved, Toast.LENGTH_SHORT).show();
        }
        return false;
    }

}
