/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack;

import android.content.Intent;
import android.os.Bundle;
import android.support.v7.widget.Toolbar;

import com.applandeo.materialcalendarview.CalendarView;
import com.applandeo.materialcalendarview.EventDay;
import com.applandeo.materialcalendarview.listeners.OnDayClickListener;
import com.applandeo.materialcalendarview.listeners.OnNavigationButtonClickListener;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import edu.uoc.sporttrack.database.SessionDatabaseHelper;
import edu.uoc.sporttrack.model.Session;
import edu.uoc.sporttrack.model.SportDay;
import edu.uoc.sporttrack.utils.Utils;

public class HistoryActivity extends BaseDrawerActivity
{
    private SessionDatabaseHelper db;
    private CalendarView logsCalendar;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        // Retrieves the content view that renders the activity.
        setContentView(R.layout.history_activity);

        // Renders the toolbar in activity
        Toolbar toolbar = renderToolbar(false, R.string.history_title);

        // Access is from menu
        renderNavigationDrawer(toolbar);

        // Creates database helper
        db = new SessionDatabaseHelper(this);

        displayCalendar();
    }

    /**
     * Display calendar with logs
     */
    private void displayCalendar()
    {
        logsCalendar = findViewById(R.id.historyCalendar);

        // Retrieves logs for the month
        List<EventDay> events = getMonthEvents(logsCalendar.getCurrentPageDate());

        // Sets logs in calendar
        logsCalendar.setEvents(events);

        logsCalendar.setOnDayClickListener(new OnDayClickListener()
        {
            @Override
            public void onDayClick(EventDay eventDay)
            {
                // Starts activity and waits for response
                Bundle bundle = new Bundle();
                bundle.putSerializable(Utils.LOGDAY, eventDay.getCalendar().getTime());
                Intent intent = new Intent(getApplicationContext(), HistoryDayActivity.class);
                intent.putExtra(Utils.BUNDLE, bundle);
                startActivityForResult(intent, Utils.CODE_SELECT_LOG_DAY);
            }
        });

        logsCalendar.setOnPreviousButtonClickListener(new OnNavigationButtonClickListener()
        {
            @Override
            public void onClick()
            {
                List<EventDay> events = getMonthEvents(logsCalendar.getCurrentPageDate());
                logsCalendar.setEvents(events);
            }
        });

        logsCalendar.setOnForwardButtonClickListener(new OnNavigationButtonClickListener()
        {
            @Override
            public void onClick()
            {
                List<EventDay> events = getMonthEvents(logsCalendar.getCurrentPageDate());
                logsCalendar.setEvents(events);
            }
        });
    }

    private List<EventDay> getMonthEvents(Calendar from)
    {
        // Gets first and last day of month
        Calendar to = Calendar.getInstance();
        to.setTime(from.getTime());
        to.set(Calendar.DAY_OF_MONTH, to.getActualMaximum(Calendar.DAY_OF_MONTH));

        List<SportDay> sportDays = new ArrayList<>();

        // Retrieves other sports logs for month
        for (Session session : db.getSessions(from.getTime(), to.getTime()))
        {
            Calendar day = Utils.toCalendar(session.getStart());
            SportDay sportDay = new SportDay(session.getIdSport(), day, R.drawable.log_sporttrack);

            if (!sportDays.contains(sportDay))
            {
                switch ((int) session.getIdSport())
                {
                    case Utils.SPORT_CYCLING:

                        sportDay.setImage(R.drawable.log_cycling);
                        break;

                    case Utils.SPORT_MOUNTBIKING:

                        sportDay.setImage(R.drawable.log_moutbiking);
                        break;

                    case Utils.SPORT_ROWING:

                        sportDay.setImage(R.drawable.log_rowing);
                        break;

                    case Utils.SPORT_RUNNING:

                        sportDay.setImage(R.drawable.log_running);
                        break;

                    case Utils.SPORT_SKIING:

                        sportDay.setImage(R.drawable.log_skiing);
                        break;

                    case Utils.SPORT_TIMED:

                        sportDay.setImage(R.drawable.log_timed);
                        break;

                    case Utils.SPORT_WORKOUT:

                        sportDay.setImage(R.drawable.log_workout);
                        break;
                }

                sportDays.add(sportDay);
            }
            else
            {
                SportDay sameDay = sportDays.get(sportDays.indexOf(sportDay));

                if (sameDay.getIdSport() != sportDay.getIdSport())
                {
                    sameDay.setImage(R.drawable.log_sporttrack);
                }
            }
        }

        // Adds events to calendar
        List<EventDay> events = new ArrayList<>();
        for (SportDay sportDay : sportDays)
        {
            events.add(new EventDay(sportDay.getDay(), sportDay.getImage()));
        }

        return events;
    }

}
