/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack;

import android.os.Bundle;
import android.widget.TextView;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.LatLng;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.sporttrack.database.SessionDatabaseHelper;
import edu.uoc.sporttrack.model.Point;
import edu.uoc.sporttrack.model.Session;
import edu.uoc.sporttrack.utils.MapUtils;
import edu.uoc.sporttrack.utils.Utils;

public class HistoryDayMapActivity extends BaseActivity implements OnMapReadyCallback
{
    private List<Point> points;
    private Session session;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        // Retrieves the content view that renders the activity.
        setContentView(R.layout.history_day_map_activity);

        // Creates database helper
        SessionDatabaseHelper db = new SessionDatabaseHelper(this);

        // Retrieves data from intent
        Bundle bundle = getIntent().getBundleExtra(Utils.BUNDLE);
        if (bundle != null)
        {
            session = bundle.getParcelable(Utils.SESSION);
        }

        if (session != null)
        {
            points = db.getAllMapPoints(session.getId());
        }

        // Renders the toolbar in activity
        renderToolbar(true, R.string.history_route);

        // Builds the map and we get notified when the map is ready to be used
        SupportMapFragment mapFragment = (SupportMapFragment) getSupportFragmentManager().findFragmentById(R.id.map);
        mapFragment.getMapAsync(this);

        // Shows route data
        displayData();
    }

    /**
     * Manipulates the map when available.
     */
    @Override
    public void onMapReady(GoogleMap map)
    {
        if (map != null)
        {
            // Moves camera to either default or currect location
            map.moveCamera(CameraUpdateFactory.newCameraPosition(MapUtils.DEFAULT_LOCATION));

            if (points != null)
            {
                // Creates route points
                List<LatLng> routePoints = new ArrayList<>();
                for (Point point : points)
                {
                    routePoints.add(new LatLng(point.getLatitude(), point.getLongitude()));
                }

                // Moves the camara and draws the route
                if (routePoints.size() > 0)
                {
                    map.moveCamera(CameraUpdateFactory.newLatLng(routePoints.get(0)));
                    MapUtils.drawMapLine(map, routePoints, null, getResources().getColor(R.color.colorPrimary));
                }
            }
        }
    }

    /**
     * Displays route data
     */
    public void displayData()
    {
        TextView mapsTime = findViewById(R.id.mapsTime);
        TextView mapsDistance = findViewById(R.id.mapsDistance);
        TextView mapSpeed = findViewById(R.id.mapsSpeed);

        String time = session.getFinish() != null ? Utils.formatTime(session.getStart(), session.getFinish()) : "";
        String speed = Utils.formatNumber(session.getSpeed()) + " " + session.getUnit() + "/h";
        String distance = Utils.formatNumber(session.getDistance()) + " " + session.getUnit();

        mapsTime.setText(time);
        mapSpeed.setText(speed);
        mapsDistance.setText(distance);
    }

}
