/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack;

import android.content.Intent;
import android.os.Bundle;
import android.support.constraint.ConstraintLayout;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;

import edu.uoc.sporttrack.database.DatabaseHelper;
import edu.uoc.sporttrack.database.RoutineDatabaseHelper;
import edu.uoc.sporttrack.model.Routine;
import edu.uoc.sporttrack.model.Sport;
import edu.uoc.sporttrack.utils.MapUtils;
import edu.uoc.sporttrack.utils.Utils;

public class MainActivity extends BaseDrawerActivity implements OnMapReadyCallback
{
    private static final String TAG = MainActivity.class.getSimpleName();

    private Sport sport;
    private ConstraintLayout fabSelect;
    private RoutineDatabaseHelper db;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        // Creates database if it doesn't exist
        createAppDatabase();

        // Retrieves the content view that renders the activity.
        setContentView(R.layout.main_activity);

        // Creates database helper
        db = new RoutineDatabaseHelper(this);

        // Renders the toolbar in activity
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        // Renders the navigation drawer
        renderNavigationDrawer(toolbar);

        // Builds the map and we get notified when the map is ready to be used
        SupportMapFragment mapFragment = (SupportMapFragment) getSupportFragmentManager().findFragmentById(R.id.map);
        mapFragment.getMapAsync(this);

    }

    // Renders the floating action button_red in activity
    private void createFloatingButtons()
    {
        // Sets texts and icons in Select Activity button
        TextView fabText = findViewById(R.id.fab_select_text);
        fabText.setText((sport != null ? sport.getName() : getResources().getText(R.string.action_select_activity)));

        ImageView fabIcon = findViewById(R.id.fab_select_icon);
        fabIcon.setImageResource(sport != null ? sport.getIcon() : R.drawable.ic_sporttrack);

        // Configures button's listeners
        if (fabSelect == null)
        {
            fabSelect = findViewById(R.id.fab_select);
            fabSelect.setOnClickListener(new View.OnClickListener()
            {
                @Override
                public void onClick(View view)
                {
                    // Starts activity and waits for response
                    Bundle bundle = new Bundle();
                    bundle.putBoolean(Utils.ACCESS_MODE, Utils.MODE_SELECTION);
                    Intent intent = new Intent(getApplicationContext(), SportActivity.class);
                    intent.putExtra(Utils.BUNDLE, bundle);
                    startActivityForResult(intent, Utils.CODE_SELECT_SPORT);
                }
            });
        }

        // Configures Start button
        View fabStart = findViewById(R.id.fab_start);
        fabStart.setVisibility(sport != null ? View.VISIBLE : View.INVISIBLE);

        if (sport != null)
        {
            switch ((int) sport.getId())
            {
                case Utils.SPORT_CYCLING:
                case Utils.SPORT_MOUNTBIKING:
                case Utils.SPORT_ROWING:
                case Utils.SPORT_RUNNING:
                case Utils.SPORT_SKIING:

                    fabStart.setOnClickListener(new View.OnClickListener()
                    {
                        @Override
                        public void onClick(View view)
                        {
                            // Starts activity and waits for response
                            Bundle bundle = new Bundle();
                            bundle.putLong(Utils.SPORT, sport.getId());
                            Intent intent = new Intent(getApplicationContext(), MapActivity.class);
                            intent.putExtra(Utils.BUNDLE, bundle);
                            startActivityForResult(intent, Utils.CODE_SELECT_GPS);
                        }
                    });

                    break;

                case Utils.SPORT_TIMED:

                    fabStart.setOnClickListener(new View.OnClickListener()
                    {
                        @Override
                        public void onClick(View view)
                        {
                            // Starts activity and waits for response
                            Bundle bundle = new Bundle();
                            bundle.putLong(Utils.SPORT, sport.getId());
                            Intent intent = new Intent(getApplicationContext(), TimedActivity.class);
                            intent.putExtra(Utils.BUNDLE, bundle);
                            startActivityForResult(intent, Utils.CODE_SELECT_TIMED);
                        }
                    });

                    break;

                case Utils.SPORT_WORKOUT:

                    // Gets current routine
                    final Routine routine = db.getCurrent();

                    fabStart.setOnClickListener(new View.OnClickListener()
                    {
                        @Override
                        public void onClick(View view)
                        {
                            if (routine != null)
                            {
                                // Starts activity and waits for response
                                Bundle bundle = new Bundle();
                                bundle.putParcelable(Utils.ROUTINE, routine);
                                bundle.putLong(Utils.SPORT, sport.getId());
                                Intent intent = new Intent(getApplicationContext(), SessionCurrentRoutineActivity.class);
                                intent.putExtra(Utils.BUNDLE, bundle);
                                startActivityForResult(intent, Utils.CODE_SELECT_CURRENT);
                            }
                            else
                            {
                                Toast.makeText(getApplicationContext(), R.string.err_current_selected, Toast.LENGTH_SHORT).show();
                            }
                        }
                    });

                    break;
            }
        }
    }

    /**
     * Callback method to get result from other activity.
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data)
    {
        super.onActivityResult(requestCode, resultCode, data);

        if (requestCode == Utils.CODE_SELECT_SPORT)
        {
            if (data != null)
            {
                sport = data.getParcelableExtra(Utils.SPORT);
            }
        }
    }

    @Override
    protected void onResume()
    {
        super.onResume();

        // Retrieves data from intent
        Bundle bundle = getIntent().getBundleExtra(Utils.BUNDLE);
        if (bundle != null)
        {
            sport = bundle.getParcelable(Utils.SPORT);
        }

        // Renders buttons on resuming
        createFloatingButtons();
    }

    /**
     * Manipulates the map when available.
     */
    @Override
    public void onMapReady(GoogleMap googleMap)
    {
        if (googleMap != null)
        {
            // Moves camera to either default or currect location
            googleMap.moveCamera(CameraUpdateFactory.newCameraPosition(MapUtils.DEFAULT_LOCATION));
        }
    }

    /**
     * Checks if database is created and if not, it creates a database from the file from the assets folder.
     */
    private void createAppDatabase()
    {
        DatabaseHelper helper = new DatabaseHelper(this);

        try
        {
            helper.createDatabase();
        }
        catch (Exception e)
        {
            Log.e(TAG, "Unable to create database: " + e);
        }
    }
}
