/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack;

import android.Manifest;
import android.content.Intent;
import android.location.Location;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.support.annotation.NonNull;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationCallback;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationResult;
import com.google.android.gms.location.LocationServices;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Polyline;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;

import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.concurrent.TimeUnit;

import edu.uoc.sporttrack.database.SessionDatabaseHelper;
import edu.uoc.sporttrack.database.SettingsDatabaseHelper;
import edu.uoc.sporttrack.model.Point;
import edu.uoc.sporttrack.model.Session;
import edu.uoc.sporttrack.model.Settings;
import edu.uoc.sporttrack.utils.MapUtils;
import edu.uoc.sporttrack.utils.Utils;

public class MapActivity extends BaseActivity implements OnMapReadyCallback
{
    private static final String TAG = MapActivity.class.getSimpleName();
    private static final long UPDATE_PARAMS_FREQUENCY = 100;
    private static final long UPDATE_MAP_FREQUENCY = 1000;
    private static final double MAX_BEARING_DIFFERENCE = 10.0;
    private static final double MIN_DRAW = 1;

    private FusedLocationProviderClient fusedLocationClient;
    private LocationRequest locationRequest;
    private LocationCallback locationCallback;
    private boolean locationUpdatesEnabled;
    private Location current;
    private Location lastDrawLocation;
    private ArrayList<LatLng> routePoints;
    private Polyline routeLine;
    private View mapView;
    private GoogleMap map;
    private float distanceToLastDraw = 0.0f;

    private LinkedList<Location> locations = new LinkedList<>();
    private double lastPoint = -1;
    private Location pending;
    private SessionDatabaseHelper db;
    private Settings settings;
    private TextView mapsTime, mapsDistance, mapSpeed;
    private Session session;
    private Handler sessionHandler, mapHandler;
    private Runnable sessionRunnable, mapRunnable;
    private int stopFlag = 100;
    private long sport = 0;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        // Retrieves the content view that renders the activity.
        setContentView(R.layout.history_day_map_activity);

        // Gets sports array from resources
        String[] sports = getResources().getStringArray(R.array.sports);

        // Retrieves data from intent
        if (getIntent().getBundleExtra(Utils.BUNDLE) != null)
        {
            sport = getIntent().getBundleExtra(Utils.BUNDLE).getLong(Utils.SPORT);
        }

        // Creates database helper
        db = new SessionDatabaseHelper(this);
        settings = new SettingsDatabaseHelper(this).get();

        // Renders the toolbar in activity
        renderToolbar(false, sports[(int) sport - 1]);

        // Checks for app permissions on runtime for this activity
        MapUtils.requestAllPermissions(this);

        // Enables receiving location updates
        enableLocationUpdates();

        // Obtain the SupportMapFragment and get notified on onMapReady method when the map is
        // ready to be used.
        SupportMapFragment mapFragment = ((SupportMapFragment) getSupportFragmentManager().findFragmentById(R.id.map));
        mapView = mapFragment.getView();
        mapFragment.getMapAsync(this);

        // Starts configuration for activity
        startActivity();
    }

    @Override
    public void onBackPressed()
    {
        super.onBackPressed();
        finishMapActivity();
    }

    @Override
    protected void onResume()
    {
        super.onResume();

        if (locationUpdatesEnabled)
        {
            startLocationUpdates();
        }
    }

    @Override
    protected void onPause()
    {
        super.onPause();
        stopLocationUpdates();
    }

    /**
     * Sets up the action bar.
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        // Inflates the menu, adding items to the action bar
        getMenuInflater().inflate(R.menu.toolbar_stop, menu);

        return true;
    }

    /**
     * Called whenever an item in action bar is selected.
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        switch (item.getItemId())
        {

            // If stop icon is clicked
            case R.id.action_stop:

                closeSession();
                return true;

            default:

                return super.onOptionsItemSelected(item);
        }
    }

    /**
     * Callback method to get result from other activity.
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data)
    {
        super.onActivityResult(requestCode, resultCode, data);

        switch (requestCode)
        {
            case Utils.CODE_SUMMARY:

                // Finishes activity and sends OK code to waiting activity
                Intent intent = new Intent();
                setResult(Utils.RESULT_OK, intent);
                finish();
                break;
        }
    }

    /**
     * Manipulates the map when available. This callback is triggered when the map is ready to be
     * used.
     */
    @Override
    public void onMapReady(GoogleMap googleMap)
    {
        map = googleMap;

        if (map != null)
        {
            // If the location permission is granted, enables the My Location layer and the
            // related control on the map.
            if (MapUtils.checkPermission(this, Manifest.permission.ACCESS_FINE_LOCATION))
            {
                map.setMyLocationEnabled(true);

                if (mapView != null && mapView.findViewById(Integer.parseInt("1")) != null)
                {
                    // Gets the location button
                    View locationButton = ((View) mapView.findViewById(Integer.parseInt("1")).getParent()).findViewById(Integer.parseInt("2"));

                    // Places the button on bottom right
                    RelativeLayout.LayoutParams layoutParams = (RelativeLayout.LayoutParams) locationButton.getLayoutParams();
                    layoutParams.addRule(RelativeLayout.ALIGN_PARENT_TOP, 0);
                    layoutParams.addRule(RelativeLayout.ALIGN_PARENT_BOTTOM, RelativeLayout.TRUE);
                    layoutParams.setMargins(0, 0, 30, 200);
                }
            }

            // Moves camera to either default or currect location
            map.moveCamera(CameraUpdateFactory.newCameraPosition(MapUtils.DEFAULT_LOCATION));

            // Moves camera to current location if available
            if (current != null)
            {
                map.moveCamera(CameraUpdateFactory.newLatLng(new LatLng(current.getLatitude(), current.getLongitude())));
            }
        }
    }

    /**
     * Handles the result of the request for location permissions.
     */
    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String permissions[], @NonNull int[] grantResults)
    {
        switch (requestCode)
        {
            case MapUtils.LOCATION_PERMISSION_REQUEST_CODE:
            {
                if (!MapUtils.isPermissionGranted(permissions, grantResults, Manifest.permission.ACCESS_FINE_LOCATION))
                {
                    // Shows dialog after permission has been denied
                    Utils.showDialog(this, null, R.string.dialog_location_request_title, R.string.dialog_location_request_denied, -1, -1,
                                     R.string.action_ok);
                }
            }
        }
    }

    /**
     * Enables location updates
     */
    private void enableLocationUpdates()
    {
        // Creates entry point to the Fused Location Provider.
        fusedLocationClient = LocationServices.getFusedLocationProviderClient(this);

        // Creates callback to manage location updates
        locationCallback = new LocationCallback()
        {
            @Override
            public void onLocationResult(LocationResult locationResult)
            {
                super.onLocationResult(locationResult);
                onLocationChanged(locationResult.getLastLocation());
            }
        };

        locationRequest = MapUtils.createLocationRequest();

        // Gets last known location
        getLastLocation();

        locationUpdatesEnabled = true;
    }

    /**
     * Gets last known recent location.
     */
    private void getLastLocation()
    {
        if (MapUtils.checkPermission(this, Manifest.permission.ACCESS_FINE_LOCATION))
        {
            try
            {
                // Listener to handle success and failure
                fusedLocationClient.getLastLocation().addOnCompleteListener(new OnCompleteListener<Location>()
                {
                    @Override
                    public void onComplete(@NonNull Task<Location> task)
                    {
                        if (task.isSuccessful() && task.getResult() != null)
                        {
                            current = task.getResult();
                        }
                        else
                        {
                            Log.w(TAG, "Failed to get last location.");
                        }
                    }
                });
            }
            catch (SecurityException e)
            {
                Log.e(TAG, "Lost location permission." + e);
            }
        }
    }

    /**
     * Starts locations updates.
     */
    private void startLocationUpdates()
    {
        if (MapUtils.checkPermission(this, Manifest.permission.ACCESS_FINE_LOCATION))
        {
            fusedLocationClient.requestLocationUpdates(locationRequest, locationCallback, Looper.myLooper());
        }
    }

    /**
     * Stops locations updates.
     */
    private void stopLocationUpdates()
    {
        fusedLocationClient.removeLocationUpdates(locationCallback);
    }

    /**
     * Sets fields in activity
     */
    private void startActivity()
    {
        // Gets fields in activity
        mapsTime = findViewById(R.id.mapsTime);
        mapsDistance = findViewById(R.id.mapsDistance);
        mapSpeed = findViewById(R.id.mapsSpeed);

        // Creates session
        session = new Session();
        session.setStart(new Date());
        session.setIdSport(sport);
        session.setUnit(settings.getDistanceUnit());
        long id = db.add(session);
        session.setId(id);

        routePoints = new ArrayList<>();

        // Creates sessionHandler to display current values
        createHandlers();
    }

    /**
     * Called when receiving a new location.
     */
    public void onLocationChanged(Location location)
    {
        if (location != null)
        {
            locations.add(location);

            // Update session with new location
            updateSessionParams(location);

            // Sets new location to current
            current = location;
            LatLng latLng = new LatLng(current.getLatitude(), current.getLongitude());

            if (lastDrawLocation != null)
            {
                distanceToLastDraw = lastDrawLocation.distanceTo(current);
            }

            if (map != null)
            {
                // Moves to new location
                map.moveCamera(CameraUpdateFactory.newLatLng(latLng));

                if (lastDrawLocation == null || distanceToLastDraw > MIN_DRAW)
                {
                    // If the distance from the last drawing is bigger than MIN_DRAW,
                    // we draw the line on map
                    drawLine(latLng);
                }
            }
        }
    }

    /**
     * Draws line to point latLng on map
     */
    private void drawLine(LatLng latLng)
    {
        routePoints.add(latLng);
        routeLine = MapUtils.drawMapLine(map, routePoints, routeLine, getResources().getColor(R.color.colorPrimary));

        lastDrawLocation = current;
        distanceToLastDraw = 0f;
    }

    /**
     * The session parameters are updated with a new location.
     */
    private void updateSessionParams(Location location)
    {
        // Sets stop flag to max seconds again
        stopFlag = 20;

        float distanceToPrevious = 0.0f;
        if (current != null)
        {
            distanceToPrevious = location.distanceTo(current);
        }
        session.setSpeed(location.getSpeed());
        session.setDistance(session.getDistance() + distanceToPrevious);
    }

    /**
     * Creates sessionHandler to display session values every second.
     */
    private void createHandlers()
    {
        // Creates sessionHandler to display current values in activity
        sessionHandler = new Handler();
        sessionRunnable = new Runnable()
        {
            @Override
            public void run()
            {
                displaySessionParams();
                sessionHandler.postDelayed(this, UPDATE_PARAMS_FREQUENCY);
            }
        };
        sessionHandler.postDelayed(sessionRunnable, UPDATE_PARAMS_FREQUENCY);

        mapHandler = new Handler();
        mapRunnable = new Runnable()
        {
            @Override
            public void run()
            {
                updateMapPoints();
                mapHandler.postDelayed(this, UPDATE_MAP_FREQUENCY);
            }
        };
        mapHandler.postDelayed(mapRunnable, UPDATE_MAP_FREQUENCY);
    }

    /**
     * Update current session params in activity.
     */
    private void displaySessionParams()
    {
        // If the stop flag seconds have passes with no updates it's because we stopped
        if (stopFlag < 1)
        {
            session.setSpeed(0.0f);
        }

        String time = Utils.formatTime(session.getStart(), new Date());
        String speed = Utils.getFormattedSpeed(session.getSpeed(), session.getUnit()) + " " + session.getUnit() + "/h";
        String distance = Utils.getFormattedDistance(session.getDistance(), session.getUnit());

        mapsTime.setText(time);
        mapSpeed.setText(speed);
        mapsDistance.setText(distance);

        stopFlag--;
    }

    /**
     * .
     */
    private void updateMapPoints()
    {
        if (locations.size() > 0 && session != null)
        {
            // Gets first location and removes from list
            Location location = locations.getFirst();
            locations.removeFirst();

            // If there is a considerable bearing deviation
            if (lastPoint == -1 || Utils.compareBearings(location.getBearing(), lastPoint) > MAX_BEARING_DIFFERENCE)
            {
                // Saves map point from session and location
                db.add(Point.createPoint(session, location));
                lastPoint = location.getBearing();
                pending = null;
            }
            else
            {
                pending = location;
            }
        }
    }

    /**
     * Asks the user if wants to stop the current session
     */
    private void closeSession()
    {

        // Creates method to be called when confirming permission request
        Runnable runnable = new Runnable()
        {
            @Override
            public void run()
            {
                // Kills handlers and finishes activity
                finishMapActivity();

                if (session != null)
                {
                    // Starts activity and waits for response
                    Bundle bundle = new Bundle();
                    bundle.putParcelable(Utils.SESSION, session);
                    Intent intent = new Intent(getApplicationContext(), SessionSummaryActivity.class);
                    intent.putExtra(Utils.BUNDLE, bundle);
                    startActivityForResult(intent, Utils.CODE_SUMMARY);
                }
            }
        };

        Utils.showDialog(this, runnable, R.string.workout_finish_title, R.string.msg_workout_finish, -1, R.string.action_yes, R.string.action_no);
    }

    /**
     * Kills handlers and finishes activity
     */
    private void finishMapActivity()
    {
        sessionHandler.removeCallbacks(sessionRunnable);
        mapHandler.removeCallbacks(mapRunnable);

        if (session != null)
        {
            session.setFinish(new Date());
            session.setDistance(Utils.convertDistance(session.getDistance(), session.getUnit()));

            long duration = session.getFinish().getTime() - session.getStart().getTime();
            long secs = TimeUnit.MILLISECONDS.toSeconds(duration);
            session.setSpeed((session.getDistance() / secs) * 3600);

            db.update(session);

            if (pending != null)
            {
                // Saves las map point
                db.add(Point.createPoint(session, pending));
            }
        }
    }

}
