/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack;

import android.content.Intent;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.EditText;
import android.widget.Toast;

import edu.uoc.sporttrack.database.PlanDatabaseHelper;
import edu.uoc.sporttrack.model.Plan;
import edu.uoc.sporttrack.utils.Utils;

public class PlanEditActivity extends BaseActivity
{
    private PlanDatabaseHelper db;
    private EditText setsEditText, repsEditText, restEditText;
    private Plan plan;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        // Retrieves the content view that renders the activity.
        setContentView(R.layout.plan_edit_activity);

        // Retrieves data from intent
        Bundle bundle = getIntent().getBundleExtra(Utils.BUNDLE);
        if (bundle != null)
        {
            plan = bundle.getParcelable(Utils.PLAN);
        }

        // Creates database helper
        db = new PlanDatabaseHelper(this);

        // Renders the toolbar in activity
        renderToolbar(true, plan != null ? plan.getExerciseName() : getResources().getString(R.string.plan_title_edit));

        setupInitialValues();
    }

    /**
     * Sets up the action bar.
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        // Inflates the menu, adding items to the action bar
        getMenuInflater().inflate(R.menu.toolbar_save, menu);
        return true;
    }

    /**
     * Called whenever an item in action bar is selected.
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        switch (item.getItemId())
        {
            // When clicking on saving an plan
            case R.id.action_save:

                // Gets value from fields
                setsEditText = findViewById(R.id.planSets);
                repsEditText = findViewById(R.id.planReps);
                restEditText = findViewById(R.id.planRest);

                // If validations are ok, plan is saved
                if (savePlan())
                {
                    // Finishes activity and sends OK code to waiting activity
                    Intent intent = new Intent();
                    setResult(Utils.RESULT_OK, intent);
                    finish();
                }
                return true;

            default:
                return super.onOptionsItemSelected(item);
        }
    }

    /**
     * Sets up initial values.
     */
    private void setupInitialValues()
    {
        if (plan != null)
        {
            setsEditText = findViewById(R.id.planSets);
            setsEditText.setText(String.valueOf(plan.getSets()));

            repsEditText = findViewById(R.id.planReps);
            repsEditText.setText(String.valueOf(plan.getReps()));

            restEditText = findViewById(R.id.planRest);
            restEditText.setText(String.valueOf(plan.getRest()));
        }
    }

    /**
     * Validates and saves plan in database and returns true if ok
     */
    private boolean savePlan()
    {
        if (validateMandatoryFields(setsEditText, repsEditText, restEditText))
        {
            if (plan == null)
            {
                plan = new Plan();
            }

            plan.setSets(Integer.parseInt(setsEditText.getText().toString()));
            plan.setReps(Integer.parseInt(repsEditText.getText().toString()));
            plan.setRest(Integer.parseInt(restEditText.getText().toString()));

            if (db.update(plan) > 0)
            {
                return true;
            }

            Toast.makeText(this, R.string.err_plan_saved, Toast.LENGTH_SHORT).show();
        }
        return false;
    }

}
