/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack;

import android.content.DialogInterface;
import android.os.Bundle;
import android.support.design.widget.TextInputLayout;
import android.support.v7.app.AlertDialog;
import android.support.v7.widget.Toolbar;
import android.text.InputFilter;
import android.text.InputType;
import android.view.Gravity;
import android.view.View;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;

import edu.uoc.sporttrack.database.ProfileDatabaseHelper;
import edu.uoc.sporttrack.database.SettingsDatabaseHelper;
import edu.uoc.sporttrack.model.Profile;
import edu.uoc.sporttrack.model.Settings;

public class ProfileActivity extends BaseDrawerActivity
{
    private ProfileDatabaseHelper profileDB;
    private SettingsDatabaseHelper settingsDB;
    private TextView nameTextView, dobTextView, genderTextView, heightTextView, weightTextView;
    private Profile profile;
    private Settings settings;
    private String[] genders;

    private SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy");
    private DecimalFormat decimalFormat = new DecimalFormat("#.##");

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        // Retrieves the content view that renders the activity.
        setContentView(R.layout.profile_activity);

        // Renders the toolbar in activity
        Toolbar toolbar = renderToolbar(false, R.string.profile_title);

        // Renders the navigation drawer
        renderNavigationDrawer(toolbar);

        // Creates database helper
        profileDB = new ProfileDatabaseHelper(this);
        settingsDB = new SettingsDatabaseHelper(this);

        // Gets profile fields
        nameTextView = findViewById(R.id.name);
        dobTextView = findViewById(R.id.dob);
        genderTextView = findViewById(R.id.gender);
        heightTextView = findViewById(R.id.height);
        weightTextView = findViewById(R.id.weight);

        // Retrieves data
        profile = profileDB.get();
        settings = settingsDB.get();

        displayProfile();
    }

    /**
     * Retrieves all data and shows in activity
     */
    private void displayProfile()
    {
        // Gets array from resources
        genders = getResources().getStringArray(R.array.genders);

        if (profile != null)
        {
            // If height unit is set, it's added to value
            String height = profile.getHeight() != null ? decimalFormat.format(profile.getHeight()) : "";
            if (settings != null && !height.isEmpty())
            {
                height += " " + settings.getHeightUnit();
            }
            // If weight unit is set, it's added to value
            String weight = profile.getWeight() != null ? decimalFormat.format(profile.getWeight()) : "";
            if (settings != null && !weight.isEmpty())
            {
                weight += " " + settings.getWeightUnit();
            }

            nameTextView.setText(profile.getName());
            dobTextView.setText(profile.getDob() != null ? dateFormat.format(profile.getDob()) : null);
            heightTextView.setText(height);
            weightTextView.setText(weight);

            // If gender has a correct value
            if (profile.getGender() > -1 && profile.getGender() < genders.length)
            {
                genderTextView.setText(genders[profile.getGender()]);
            }
        }
        else
        {
            profile = new Profile();
            profile.setGender(-1);
        }
    }

    /**
     * Opens dialog to update name
     */
    public void onNameClicked(View view)
    {
        // Sets initial value
        final EditText editText = new EditText(this);
        editText.setFilters(new InputFilter[]{new InputFilter.LengthFilter(50)});
        editText.setText(profile != null ? profile.getName() : null);
        editText.setSelectAllOnFocus(true);

        // Shows dialog to enter name
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(getResources().getString(R.string.profile_name));
        builder.setView(editText);
        builder.setPositiveButton(getResources().getString(R.string.action_ok), new DialogInterface.OnClickListener()
        {
            @Override
            public void onClick(DialogInterface dialog, int which)
            {
                String name = String.valueOf(editText.getText());
                if (name.trim().isEmpty())
                {
                    // If no nameTextView is informed, an error is shown
                    Toast.makeText(getApplicationContext(), getResources().getString(R.string.err_required_field), Toast.LENGTH_SHORT).show();
                }
                else
                {
                    profile.setName(name);
                    updateProfile();
                }
            }
        });
        builder.setNegativeButton(getResources().getString(R.string.action_cancel), null);
        builder.show();
    }

    /**
     * Opens dialog to update date of birth
     */
    public void onDobClicked(View view)
    {
        final DatePicker datePicker = new DatePicker(this);

        // Sets initial value
        if (profile != null && profile.getDob() != null)
        {
            Calendar cal = Calendar.getInstance();
            cal.setTime(profile.getDob());
            datePicker.updateDate(cal.get(Calendar.YEAR), cal.get(Calendar.MONTH), cal.get(Calendar.DAY_OF_MONTH));
        }

        // Shows dialog to display datepicker
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(getResources().getString(R.string.profile_dob));
        builder.setView(datePicker);
        builder.setPositiveButton(getResources().getString(R.string.action_ok), new DialogInterface.OnClickListener()
        {
            @Override
            public void onClick(DialogInterface dialog, int which)
            {
                int year = datePicker.getYear();
                int mon = datePicker.getMonth();
                int day = datePicker.getDayOfMonth();

                profile.setDob(new GregorianCalendar(year, mon, day).getTime());
                updateProfile();
            }
        });
        builder.setNegativeButton(getResources().getString(R.string.action_cancel), null);
        builder.show();
    }

    /**
     * Opens dialog to update genderTextView
     */
    public void onGenderClicked(View view)
    {
        // Shows dialog to enter genderTextView
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(getResources().getString(R.string.profile_gender));
        builder.setSingleChoiceItems(genders, profile.getGender(), new DialogInterface.OnClickListener()
        {
            @Override
            public void onClick(DialogInterface dialog, int which)
            {
                profile.setGender(which);
                updateProfile();
                dialog.dismiss();
            }
        });
        builder.show();
    }

    /**
     * Opens dialog to update heightTextView
     */
    public void onHeightClicked(View view)
    {
        // Sets initial value
        final EditText editText = new EditText(this);
        editText.setFilters(new InputFilter[]{new InputFilter.LengthFilter(5)});
        editText.setInputType(InputType.TYPE_CLASS_NUMBER | InputType.TYPE_NUMBER_FLAG_DECIMAL);
        editText.setGravity(Gravity.CENTER_HORIZONTAL);
        editText.setText(profile != null && profile.getHeight() != null ? decimalFormat.format(profile.getHeight()) : null);

        final TextInputLayout layout = new TextInputLayout(this);
        layout.setCounterEnabled(true);
        layout.setCounterMaxLength(5);
        layout.setPadding((int) getResources().getDimension(R.dimen.margin_xxxlarge), 0, (int) getResources().getDimension(R.dimen.margin_xxxlarge),
                          0);
        layout.addView(editText);

        // Gets title
        String title = getResources().getString(R.string.profile_height) + (settings != null && settings.getHeightUnit() != null ? String
                .format(" (%s)", settings.getHeightUnit()) : "");

        // Shows dialog to enter height
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(title);
        builder.setView(layout);
        builder.setPositiveButton(getResources().getString(R.string.action_ok), new DialogInterface.OnClickListener()
        {
            @Override
            public void onClick(DialogInterface dialog, int which)
            {
                String value = String.valueOf(editText.getText());
                if (value.trim().isEmpty())
                {
                    // If no value is informed, an error is shown
                    Toast.makeText(getApplicationContext(), getResources().getString(R.string.err_required_field), Toast.LENGTH_SHORT).show();
                }
                else
                {
                    profile.setHeight(Double.valueOf(value));
                    updateProfile();
                }
            }
        });
        builder.setNegativeButton(getResources().getString(R.string.action_cancel), null);
        builder.show();
    }

    /**
     * Opens dialog to update weightTextView
     */
    public void onWeightClicked(View view)
    {
        // Sets initial value
        final EditText editText = new EditText(this);
        editText.setFilters(new InputFilter[]{new InputFilter.LengthFilter(5)});
        editText.setInputType(InputType.TYPE_CLASS_NUMBER | InputType.TYPE_NUMBER_FLAG_DECIMAL);
        editText.setGravity(Gravity.CENTER_HORIZONTAL);
        editText.setText(profile != null && profile.getWeight() != null ? decimalFormat.format(profile.getWeight()) : null);

        final TextInputLayout layout = new TextInputLayout(this);
        layout.setCounterEnabled(true);
        layout.setCounterMaxLength(5);
        layout.setPadding((int) getResources().getDimension(R.dimen.margin_xxxlarge), 0, (int) getResources().getDimension(R.dimen.margin_xxxlarge),
                          0);
        layout.addView(editText);

        // Gets title
        String title = getResources().getString(R.string.profile_weight) + (settings != null && settings.getWeightUnit() != null ? String
                .format(" (%s)", settings.getWeightUnit()) : "");

        // Shows dialog to enter weight
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(title);
        builder.setView(layout);
        builder.setPositiveButton(getResources().getString(R.string.action_ok), new DialogInterface.OnClickListener()
        {
            @Override
            public void onClick(DialogInterface dialog, int which)
            {
                String value = String.valueOf(editText.getText());
                if (value.trim().isEmpty())
                {
                    // If no value is informed, an error is shown
                    Toast.makeText(getApplicationContext(), getResources().getString(R.string.err_required_field), Toast.LENGTH_SHORT).show();
                }
                else
                {
                    profile.setWeight(Double.valueOf(value));
                    updateProfile();
                }
            }
        });
        builder.setNegativeButton(getResources().getString(R.string.action_cancel), null);
        builder.show();
    }

    /**
     * Updates profile into database
     */
    private void updateProfile()
    {
        // Updates values
        if (profileDB.update(profile) > 0)
        {
            // Refreshes values in activity
            displayProfile();
        }
        else
        {
            // Shows error if update fails
            Toast.makeText(getApplicationContext(), getResources().getString(R.string.err_profile_saved), Toast.LENGTH_SHORT).show();
        }
    }
}
