/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack;

import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AlertDialog;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.sporttrack.adapter.OnItemActionListener;
import edu.uoc.sporttrack.adapter.RecyclerAdapter;
import edu.uoc.sporttrack.database.DayDatabaseHelper;
import edu.uoc.sporttrack.model.Day;
import edu.uoc.sporttrack.model.Itemizable;
import edu.uoc.sporttrack.model.Routine;
import edu.uoc.sporttrack.utils.Utils;

public class RoutineDetailActivity extends BaseActivity
{
    private Routine routine;
    private DayDatabaseHelper db;
    private RecyclerAdapter adapter;
    private List<Itemizable> items;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        // Retrieves the content view that renders the activity.
        setContentView(R.layout.routine_detail_activity);

        // Retrieves data from intent
        Bundle bundle = getIntent().getBundleExtra(Utils.BUNDLE);
        if (bundle != null)
        {
            routine = bundle.getParcelable(Utils.ROUTINE);
        }

        // Renders the toolbar in activity
        renderToolbar(true, routine != null ? routine.getName() : getResources().getString(R.string.routine_title_detail));

        // Creates database helper
        db = new DayDatabaseHelper(this);

        setupInitialValues();
    }

    /**
     * Sets up the action bar.
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        // Inflates the menu, adding items to the action bar
        getMenuInflater().inflate(R.menu.toolbar_add, menu);
        return true;
    }

    /**
     * Called whenever an item in action bar is selected.
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        switch (item.getItemId())
        {
            // If add icon is clicked
            case R.id.action_add:

                // Starts activity and waits for response
                Bundle bundle = new Bundle();
                bundle.putParcelable(Utils.ROUTINE, routine);
                Intent intent = new Intent(getApplicationContext(), DayEditActivity.class);
                intent.putExtra(Utils.BUNDLE, bundle);
                startActivityForResult(intent, Utils.CODE_ADD_DAY);

                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    /**
     * Callback method to get result from other activity.
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data)
    {
        super.onActivityResult(requestCode, resultCode, data);

        // Checks if the request code is from the called activity and result is OK
        if (resultCode == Utils.RESULT_OK)
        {
            switch (requestCode)
            {
                case Utils.CODE_ADD_DAY:

                    // Shows to the user that the action has been OK
                    Toast.makeText(this, R.string.msg_day_saved, Toast.LENGTH_SHORT).show();
                    break;

                case Utils.CODE_UPDATE_DAY:
                    // Shows to the user that the action has been OK
                    Toast.makeText(this, R.string.msg_day_updated, Toast.LENGTH_SHORT).show();
                    break;
            }
        }

        //Filters items in recycler view
        refreshItems();
    }

    /**
     * Sets up initial values.
     */
    private void setupInitialValues()
    {
        TextView desc = findViewById(R.id.routineDescription);
        desc.setText(routine != null ? routine.getDescription() : "");

        // Not showing if there is no description
        if (routine.getDescription() == null || routine.getDescription().isEmpty())
        {
            desc.setVisibility(View.GONE);
            findViewById(R.id.descTxt).setVisibility(View.GONE);
        }

        // Lists all routine days for selected routine
        displayRoutineDays();
    }

    /**
     * Lists all routine days in recycler view for received routine
     */
    private void displayRoutineDays()
    {
        // Populates recycler view with items from cursor
        RecyclerView recyclerView = findViewById(R.id.routineDaysRecyclerView);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));

        // Creates adapter for recycler view
        items = new ArrayList<>();
        adapter = new RecyclerAdapter(this, R.layout.row_general, items, this, false);
        items.addAll(db.getAll(routine.getId()));
        recyclerView.setAdapter(adapter);
    }

    /**
     * Callback method when a item in list view is clicked.
     * See {@link OnItemActionListener#onItemSelect(Itemizable)}
     */
    @Override
    public void onItemSelect(Itemizable itemizable)
    {
        // Starts activity and waits for response
        Bundle bundle = new Bundle();
        bundle.putParcelable(Utils.DAY, (Day) itemizable);
        bundle.putParcelable(Utils.ROUTINE, routine);
        Intent intent = new Intent(getApplicationContext(), DayDetailActivity.class);
        intent.putExtra(Utils.BUNDLE, bundle);
        startActivityForResult(intent, Utils.CODE_SELECT_DAY);
    }

    /**
     * Configures when a item is to be deleted.
     * See {@link OnItemActionListener#onItemDelete(Itemizable)}
     */
    @Override
    public void onItemDelete(final Itemizable itemizable)
    {
        // Shows dialog to confirm
        AlertDialog.Builder builder = new AlertDialog.Builder(RoutineDetailActivity.this);
        builder.setTitle(getResources().getString(R.string.day_title_delete));
        builder.setMessage(getResources().getString(R.string.msg_day_delete));
        builder.setCancelable(false);
        builder.setPositiveButton(getResources().getString(R.string.action_yes), new DialogInterface.OnClickListener()
        {
            @Override
            public void onClick(DialogInterface dialog, int which)
            {
                if (db.delete(itemizable.getId()) > 0)
                {
                    //Refreshes items in recycler view
                    refreshItems();

                    Toast.makeText(getApplicationContext(), getResources().getString(R.string.msg_day_deleted), Toast.LENGTH_SHORT).show();
                }
                else
                {
                    Toast.makeText(getApplicationContext(), getResources().getString(R.string.err_day_deleted), Toast.LENGTH_SHORT).show();
                }
            }
        });

        builder.setNegativeButton(getResources().getString(R.string.action_no), null);
        builder.show();
    }

    /**
     * Configures when a item is to be edited.
     * See {@link OnItemActionListener#onItemEdit(Itemizable)}
     */
    @Override
    public void onItemEdit(Itemizable itemizable)
    {
        // Starts activity and waits for response
        Bundle bundle = new Bundle();
        bundle.putParcelable(Utils.DAY, (Day) itemizable);
        bundle.putParcelable(Utils.ROUTINE, routine);
        Intent intent = new Intent(getApplicationContext(), DayEditActivity.class);
        intent.putExtra(Utils.BUNDLE, bundle);
        startActivityForResult(intent, Utils.CODE_UPDATE_DAY);
    }

    /**
     * Refreshes items in list and notifies adapter in recycler view.
     */
    private void refreshItems()
    {
        adapter.getManager().closeAllItems();
        this.items.clear();
        this.items.addAll(db.getAll(routine.getId()));
        adapter.notifyDataSetChanged();
    }

}
