/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack;

import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AlertDialog;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

import edu.uoc.sporttrack.adapter.RecyclerAdapter;
import edu.uoc.sporttrack.database.PlanDatabaseHelper;
import edu.uoc.sporttrack.database.SessionDatabaseHelper;
import edu.uoc.sporttrack.model.Day;
import edu.uoc.sporttrack.model.Itemizable;
import edu.uoc.sporttrack.model.LogPlan;
import edu.uoc.sporttrack.model.Plan;
import edu.uoc.sporttrack.model.Session;
import edu.uoc.sporttrack.utils.Utils;

public class SessionDayActivity extends BaseActivity
{
    private PlanDatabaseHelper planDB;
    private SessionDatabaseHelper sessionDB;
    private Day day = null;
    private Menu menu;
    private Thread sessionTimer;
    private Session session = null;
    private long sport;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        // Retrieves the content view that renders the activity.
        setContentView(R.layout.session_day_activity);

        // Retrieves data from intent
        Bundle bundle = getIntent().getBundleExtra(Utils.BUNDLE);
        if (bundle != null)
        {
            sport = getIntent().getBundleExtra(Utils.BUNDLE).getLong(Utils.SPORT);
            day = bundle.getParcelable(Utils.DAY);
        }

        // Renders the toolbar in activity
        renderToolbar(true, day != null ? day.getName() : getResources().getString(R.string.day_title_detail));

        // Creates database helper
        planDB = new PlanDatabaseHelper(this);
        sessionDB = new SessionDatabaseHelper(this);

        // Creates session from day data or retrieves current session if exists
        getSession();

        // Displays session time if there is one
        displaySessionTime();

        // Lists all plans for selected day
        displayPlans();

    }

    @Override
    protected void onResume()
    {
        super.onResume();

        if (session != null && session.getId() != 0)
        {
            // Creates new thread for session timer
            startSessionTimer();
        }
    }

    @Override
    protected void onPause()
    {
        super.onPause();

        if (session != null && session.getId() != 0)
        {
            // Stops thread for session timer
            stopSessionTimer();
        }
    }

    /**
     * Sets up the action bar.
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        this.menu = menu;

        // Inflates the menu, adding items to the action bar
        getMenuInflater().inflate(R.menu.toolbar_start_stop, menu);

        // Shows appropriate item in toolbar
        refreshToolbarItems();

        return true;
    }

    /**
     * Called whenever an item in action bar is selected.
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        switch (item.getItemId())
        {
            // If start icon is clicked
            case R.id.action_start:

                // If validations are ok, exercise is saved
                if (saveSession())
                {
                    // Starts activity and waits for response
                    Bundle bundle = new Bundle();
                    bundle.putParcelable(Utils.SESSION, session);
                    Intent intent = new Intent(getApplicationContext(), SessionStartActivity.class);
                    intent.putExtra(Utils.BUNDLE, bundle);
                    startActivityForResult(intent, Utils.CODE_START_WORKOUT);
                    return true;
                }

                // If stop icon is clicked
            case R.id.action_stop:

                // Stops  session
                closeSession();
                return true;

            // If add icon is clicked
            case R.id.action_add:

                // Starts activity and waits for response
                Bundle bundle = new Bundle();
                bundle.putBoolean(Utils.ACCESS_MODE, Utils.MODE_SELECTION);
                bundle.putParcelable(Utils.SESSION, session);
                bundle.putParcelable(Utils.DAY, day);
                Intent intent = new Intent(getApplicationContext(), BodypartActivity.class);
                intent.putExtra(Utils.BUNDLE, bundle);
                startActivityForResult(intent, Utils.CODE_SELECT_EXERCISE);
                return true;

            default:

                return super.onOptionsItemSelected(item);
        }
    }

    /**
     * Shows icon depending on session status
     */
    private void refreshToolbarItems()
    {
        if (session == null || session.getId() == 0)
        {
            // Shows start item
            MenuItem item = menu.findItem(R.id.action_start);
            item.setVisible(true);

            // Hides add item
            item = menu.findItem(R.id.action_add);
            item.setVisible(false);

            // Hides stop item
            item = menu.findItem(R.id.action_stop);
            item.setVisible(false);
        }
        else
        {
            // Shows stop item
            MenuItem item = menu.findItem(R.id.action_stop);
            item.setVisible(true);

            // Shows stop item
            item = menu.findItem(R.id.action_add);
            item.setVisible(true);

            // Hides start item
            item = menu.findItem(R.id.action_start);
            item.setVisible(false);
        }
    }

    /**
     * Callback method to get result from other activity.
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data)
    {
        super.onActivityResult(requestCode, resultCode, data);

        switch (requestCode)
        {
            case Utils.CODE_START_WORKOUT:

                // Retrieves session and displays plans list
                getSession();
                displayPlans();
                if (session != null)
                {
                    displaySessionTime();
                    refreshToolbarItems();
                }

                if (resultCode == Utils.RESULT_OK)
                {
                    closeSession();
                }

                break;

            case Utils.CODE_SELECT_EXERCISE:

                if (session != null)
                {
                    // Retrieves session data from database
                    retrieveSessionData();
                    displayPlans();
                    displaySessionTime();
                    refreshToolbarItems();
                }
                break;

            case Utils.CODE_SUMMARY:

                // Finishes activity and sends OK code to waiting activity
                Intent intent = new Intent();
                setResult(Utils.RESULT_OK, intent);
                finish();
                break;
        }
    }

    /**
     * Callback method when a item in list view is clicked.
     * See {@link edu.uoc.sporttrack.adapter.OnItemActionListener#onItemSelect(Itemizable)}
     */
    @Override
    public void onItemSelect(Itemizable itemizable)
    {
        // Starts activity and waits for response
        Bundle bundle = new Bundle();
        bundle.putParcelable(Utils.SESSION, session);
        bundle.putParcelable(Utils.LOGPLAN, (LogPlan) itemizable);
        bundle.putParcelable(Utils.DAY, day);
        Intent intent = new Intent(getApplicationContext(), SessionStartActivity.class);
        intent.putExtra(Utils.BUNDLE, bundle);
        startActivityForResult(intent, Utils.CODE_START_WORKOUT);
    }

    /**
     * Either creates session from day data or it retrieves from database
     */
    private void getSession()
    {
        // Retrieve current session from database
        session = sessionDB.getActiveWorkoutSession();

        if (session != null)
        {
            // Retrieves data for current session
            retrieveSessionData();
        }

        // If there is no current session, it's created
        if (day != null && session == null)
        {
            session = new Session();
            session.setIdSport(sport);
            session.setIdDay(day.getId());
            session.setDayName(day.getName());

            List<Plan> plans = planDB.getAll(day.getId());
            for (Plan plan : plans)
            {
                LogPlan logPlan = new LogPlan();
                logPlan.setIdExercise(plan.getIdExercise());
                logPlan.setExerciseName(plan.getExerciseName());
                logPlan.setBodypart(plan.getBodypart());
                logPlan.setPosition(plan.getPosition());
                logPlan.setSets(plan.getSets());
                logPlan.setReps(plan.getReps());
                logPlan.setRest(plan.getRest());
                logPlan.setIdPlan(plan.getId());
                session.getLogPlans().add(logPlan);
            }
        }
    }

    /**
     * Lists all plans in recycler view for received day
     */
    private void displayPlans()
    {
        if (session != null)
        {
            // Gets items
            List<Itemizable> items = new ArrayList<>();
            items.addAll(session.getLogPlans());

            // Creates adapter for recycler view
            RecyclerAdapter adapter = new RecyclerAdapter(this, R.layout.row_session, items, this, true);

            // Creates recycler view and binds adapter
            RecyclerView recyclerView = findViewById(R.id.sessionRecyclerView);
            recyclerView.setLayoutManager(new LinearLayoutManager(this));
            recyclerView.setAdapter(adapter);
        }
    }

    /**
     * Validates and saves exercise in database and returns true if ok
     */
    private boolean saveSession()
    {
        if (session != null)
        {
            session.setStart(new Date());
            if (sessionDB.add(session) > 0)
            {
                return true;
            }
        }

        Toast.makeText(this, R.string.err_session_start, Toast.LENGTH_SHORT).show();
        return false;
    }

    /**
     * Asks the user if wants to stop the current session
     */
    private void closeSession()
    {
        // Shows dialog to confirm
        AlertDialog.Builder builder = new AlertDialog.Builder(SessionDayActivity.this);
        builder.setTitle(getResources().getString(R.string.workout_finish_title));
        builder.setMessage(getResources().getString(R.string.msg_workout_finish));
        builder.setCancelable(false);
        builder.setPositiveButton(getResources().getString(R.string.action_yes), new DialogInterface.OnClickListener()
        {
            @Override
            public void onClick(DialogInterface dialog, int which)
            {
                if (session != null)
                {
                    // Retrieves all data
                    retrieveSessionData();

                    // Finishes the session
                    session.setFinish(new Date());
                    sessionDB.update(session);

                    // Deletes all log plans without sets
                    for (LogPlan log : session.getLogPlans())
                    {
                        if (log.getLogSets().size() == 0)
                        {
                            sessionDB.deleteLogPlan(log.getId());
                        }
                    }

                    // Starts activity and waits for response
                    Bundle bundle = new Bundle();
                    bundle.putParcelable(Utils.SESSION, session);
                    Intent intent = new Intent(getApplicationContext(), SessionSummaryActivity.class);
                    intent.putExtra(Utils.BUNDLE, bundle);
                    startActivityForResult(intent, Utils.CODE_SUMMARY);
                }
            }
        });

        builder.setNegativeButton(getResources().getString(R.string.action_no), null);
        builder.show();
    }

    /**
     * Retrieves session data from database.
     */
    private void retrieveSessionData()
    {
        // Retrieves log plans for session
        session.setLogPlans(sessionDB.getAllLogPlans(session.getId()));

        for (LogPlan log : session.getLogPlans())
        {
            // Retrieves all log sets for each log plan
            log.setLogSets(sessionDB.getAllLogSets(log.getId()));
        }
    }

    /**
     * Displays current session time
     */
    private void displaySessionTime()
    {
        View view = findViewById(R.id.sessionContainer);

        if (session != null && session.getId() != 0 && session.getStart() != null)
        {
            view.setVisibility(View.VISIBLE);

            long duration = new Date().getTime() - session.getStart().getTime();
            long diffInMinutes = TimeUnit.MILLISECONDS.toMinutes(duration);

            TextView sessionTime = findViewById(R.id.sessionTime);
            sessionTime.setText(String.valueOf(diffInMinutes));
        }
        else
        {
            view.setVisibility(View.GONE);
        }
    }

    /**
     * Starts thread to update session time
     */
    private void startSessionTimer()
    {
        sessionTimer = new Thread()
        {
            @Override
            public void run()
            {
                try
                {
                    while (!isInterrupted())
                    {
                        // Refresh timer every minute
                        Thread.sleep(60000);
                        runOnUiThread(new Runnable()
                        {
                            @Override
                            public void run()
                            {
                                displaySessionTime();
                            }
                        });
                    }
                }
                catch (InterruptedException e)
                {
                    // Does nothing
                }
            }

        };

        sessionTimer.start();
    }

    /**
     * Stops thread to update session time
     */
    private void stopSessionTimer()
    {
        if (sessionTimer != null)
        {
            sessionTimer.interrupt();
            sessionTimer = null;
        }
    }
}
