/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack;

import android.content.Intent;
import android.os.Bundle;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import edu.uoc.sporttrack.adapter.RecyclerAdapter;
import edu.uoc.sporttrack.database.PlanDatabaseHelper;
import edu.uoc.sporttrack.database.SessionDatabaseHelper;
import edu.uoc.sporttrack.database.SettingsDatabaseHelper;
import edu.uoc.sporttrack.model.Day;
import edu.uoc.sporttrack.model.Itemizable;
import edu.uoc.sporttrack.model.LogPlan;
import edu.uoc.sporttrack.model.LogSet;
import edu.uoc.sporttrack.model.Plan;
import edu.uoc.sporttrack.model.Session;
import edu.uoc.sporttrack.model.Settings;
import edu.uoc.sporttrack.utils.Utils;

public class SessionStartActivity extends BaseActivity
{
    private SessionDatabaseHelper sessionDB;
    private EditText logWeight, logReps, logRest;
    private String unit = "kg";
    private PlanDatabaseHelper planDB;
    private Session session;
    private LogPlan logPlan;
    private Day day;
    private LogSet logSet;
    private Toolbar toolbar;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        // Retrieves the content view that renders the activity.
        setContentView(R.layout.session_start_activity);

        // Creates database helper
        sessionDB = new SessionDatabaseHelper(this);
        planDB = new PlanDatabaseHelper(this);
        SettingsDatabaseHelper settingsDB = new SettingsDatabaseHelper(this);

        // Retrieves data from intent
        Bundle bundle = getIntent().getBundleExtra(Utils.BUNDLE);
        if (bundle != null)
        {
            session = bundle.getParcelable(Utils.SESSION);
            logPlan = bundle.getParcelable(Utils.LOGPLAN);
            day = bundle.getParcelable(Utils.DAY);
        }

        if (session != null)
        {
            retrieveSessionData();

            // If no log plan workout is initial
            if (logPlan == null && session.getLogPlans() != null)
            {
                // We get the first log plan
                logPlan = session.getLogPlans().get(0);
            }

            // Display las logs for this plan
            displayLastLogs();

            // Renders the toolbar in activity
            toolbar = renderToolbar(true, getToolbarTitle());

            // Gets settings
            Settings settings = settingsDB.get();
            unit = settings != null ? settings.getWeightUnit() : unit;

            setupInitialValues();
        }
    }

    /**
     * Sets up the action bar.
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        // Inflates the menu, adding items to the action bar
        getMenuInflater().inflate(R.menu.toolbar_rest, menu);
        return true;
    }

    /**
     * Called whenever an item in action bar is selected.
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        switch (item.getItemId())
        {
            // When clicking on saving an session
            case R.id.action_rest:

                logWeight = findViewById(R.id.logWeight);
                logReps = findViewById(R.id.logReps);
                logRest = findViewById(R.id.logRest);

                if (validateMandatoryFields(logWeight, logReps, logRest))
                {
                    // If session has not started yet it's created
                    if (session.getId() == 0)
                    {
                        createSession();
                    }

                    // If validations are ok, log is saved
                    if (session.getId() != 0 && saveLogSet())
                    {
                        // Retrieves session data from database
                        retrieveSessionData();

                        // Refreshes items in recycler view
                        displayLastLogs();

                        // Starts activity and waits for response
                        Bundle bundle = new Bundle();
                        bundle.putParcelable(Utils.LOGSET, logSet);
                        Intent intent = new Intent(getApplicationContext(), CounterdownActivity.class);
                        intent.putExtra(Utils.BUNDLE, bundle);
                        startActivityForResult(intent, Utils.CODE_START_COUNTER);
                    }
                }
                return true;

            default:
                return super.onOptionsItemSelected(item);
        }
    }

    /**
     * Callback method to get result from other activity.
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data)
    {
        super.onActivityResult(requestCode, resultCode, data);
        switch (requestCode)
        {
            case Utils.CODE_START_COUNTER:

                if (session != null && logPlan != null)
                {
                    // Checks if current plan is complete
                    if (logPlan.getSets() <= logPlan.getLogSets().size())
                    {
                        int planPos = session.getLogPlans().indexOf(logPlan);
                        if (planPos != -1)
                        {
                            // Checks if the log plan has complete its sets
                            if (planPos + 1 < session.getLogPlans().size())
                            {
                                // Retrieves next log plan
                                logPlan = session.getLogPlans().get(planPos + 1);
                                setupInitialValues();
                                displayLastLogs();
                            }
                            else
                            {
                                // Finishes activity and sends OK code to waiting activity
                                Intent intent = new Intent();
                                setResult(Utils.RESULT_OK, intent);
                                finish();
                            }
                        }
                    }

                    // Sets title in toolbar
                    toolbar.setTitle(getToolbarTitle());
                }
                break;
        }
    }

    /**
     * Retrieves session data from database.
     */
    private void retrieveSessionData()
    {
        // Retrieves log plans for session
        session.setLogPlans(sessionDB.getAllLogPlans(session.getId()));

        for (LogPlan log : session.getLogPlans())
        {
            // Retrieves all log sets for each log plan
            log.setLogSets(sessionDB.getAllLogSets(log.getId()));

            // Retrieves sets for current log plan
            if (logPlan != null && logPlan.getId() == log.getId())
            {
                logPlan = log;
            }
        }
    }

    /**
     * Sets up initial values.
     */
    private void setupInitialValues()
    {
        TextView weightUnit = findViewById(R.id.logWeightUnit);
        weightUnit.setText(unit);

        if (logPlan != null)
        {
            TextView exerciseName = findViewById(R.id.logExerciseName);
            exerciseName.setText(logPlan.getExerciseName());

            logReps = findViewById(R.id.logReps);
            logReps.setText(String.valueOf(logPlan.getReps()));

            logRest = findViewById(R.id.logRest);
            logRest.setText(String.valueOf(logPlan.getRest()));
        }
    }

    /**
     * Lists all logged sets for this plan
     */
    private void displayLastLogs()
    {
        View view = findViewById(R.id.logLastLogs);

        if (logPlan != null && logPlan.getLogSets() != null && logPlan.getLogSets().size() > 0)
        {
            view.setVisibility(View.VISIBLE);
            // Populates recycler view with items from cursor
            RecyclerView recyclerView = findViewById(R.id.logSetsRecyclerView);
            recyclerView.setLayoutManager(new LinearLayoutManager(this));

            // Creates adapter for recycler view
            List<Itemizable> items = new ArrayList<>();
            RecyclerAdapter adapter = new RecyclerAdapter(this, R.layout.row_logs, items, this, true);
            items.addAll(logPlan.getLogSets());
            recyclerView.setAdapter(adapter);
        }
        else
        {
            view.setVisibility(View.GONE);
        }
    }

    /**
     * Validates and saves sets in database and returns true if ok
     */
    private boolean saveLogSet()
    {
        if (validateMandatoryFields(logWeight, logReps, logRest))
        {
            if (logPlan != null)
            {
                logSet = new LogSet();
                logSet.setIdLog(logPlan.getId());
                logSet.setSetDate(new Date());
                logSet.setWeight(Double.valueOf(logWeight.getText().toString()));
                logSet.setWeightUnit(unit);
                logSet.setReps(Integer.valueOf(logReps.getText().toString()));
                logSet.setRest(Integer.valueOf(logRest.getText().toString()));

                if (sessionDB.add(logSet) > 0)
                {
                    return true;
                }
            }
            Toast.makeText(this, R.string.err_set_saved, Toast.LENGTH_SHORT).show();
        }

        return false;
    }

    /**
     * Gets toolbar title based on sets.
     */
    private String getToolbarTitle()
    {
        int currentSet = (logPlan.getLogSets() != null ? logPlan.getLogSets().size() : 0) + 1;
        return getResources().getString(R.string.workout_set) + " " + currentSet + "/" + logPlan.getSets();
    }

    /**
     * Validates and saves exercise in database
     */
    private void saveSession()
    {
        if (session != null)
        {
            session.setStart(new Date());
            if (sessionDB.add(session) > 0)
            {
                return;
            }
        }

        Toast.makeText(this, R.string.err_session_start, Toast.LENGTH_SHORT).show();
    }

    /**
     * Creates new session
     */
    private void createSession()
    {
        List<Plan> plans = planDB.getAll(day.getId());
        for (Plan plan : plans)
        {
            LogPlan logPlan = new LogPlan();
            logPlan.setIdExercise(plan.getIdExercise());
            logPlan.setExerciseName(plan.getExerciseName());
            logPlan.setBodypart(plan.getBodypart());
            logPlan.setPosition(plan.getPosition());
            logPlan.setSets(plan.getSets());
            logPlan.setReps(plan.getReps());
            logPlan.setRest(plan.getRest());
            logPlan.setIdPlan(plan.getId());
            session.getLogPlans().add(logPlan);
        }

        // Saves session
        saveSession();

        // We update current log plan
        for (LogPlan log : session.getLogPlans())
        {
            if (log.getIdPlan() == logPlan.getIdPlan())
            {
                logPlan = log;
                break;
            }
        }
    }

}
