/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack;

import android.os.Bundle;
import android.view.View;
import android.widget.TextView;

import java.util.Locale;
import java.util.concurrent.TimeUnit;

import edu.uoc.sporttrack.database.SessionDatabaseHelper;
import edu.uoc.sporttrack.database.SettingsDatabaseHelper;
import edu.uoc.sporttrack.model.LogPlan;
import edu.uoc.sporttrack.model.LogSet;
import edu.uoc.sporttrack.model.Session;
import edu.uoc.sporttrack.model.Settings;
import edu.uoc.sporttrack.utils.Utils;

public class SessionSummaryActivity extends BaseActivity
{
    private SessionDatabaseHelper db;
    private Session session = null;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        // Retrieves the content view that renders the activity.
        setContentView(R.layout.session_summary_activity);

        // Retrieves data from intent
        Bundle bundle = getIntent().getBundleExtra(Utils.BUNDLE);
        if (bundle != null)
        {
            session = bundle.getParcelable(Utils.SESSION);
        }

        // Renders the toolbar in activity
        renderToolbar(true, getResources().getString(R.string.summary_title_detail));

        // Creates database helper
        db = new SessionDatabaseHelper(this);

        // Retrieves session data
        retrieveSessionData();

        // Displays session data
        displaySummaryData();
    }

    /**
     * Retrieves session data from database.
     */
    private void retrieveSessionData()
    {
        if (session.getIdSport() == Utils.SPORT_WORKOUT)
        {
            // Retrieves log plans for session
            session.setLogPlans(db.getAllLogPlans(session.getId()));

            for (LogPlan log : session.getLogPlans())
            {
                // Retrieves all log sets for each log plan
                log.setLogSets(db.getAllLogSets(log.getId()));
            }
        }
    }

    /**
     * Displays summary data
     */
    private void displaySummaryData()
    {
        Settings settings = new SettingsDatabaseHelper(this).get();

        TextView summDate = findViewById(R.id.summDate);
        TextView summTime = findViewById(R.id.summTime);
        TextView summExercises = findViewById(R.id.summExercises);
        TextView summExercisesTxt = findViewById(R.id.summExercisesTxt);
        TextView summRest = findViewById(R.id.summRest);
        TextView summRestTxt = findViewById(R.id.summRestTxt);
        TextView summSets = findViewById(R.id.summSets);
        TextView summSetsTxt = findViewById(R.id.summSetsTxt);
        TextView summWeight = findViewById(R.id.summWeight);
        TextView summWeightTxt = findViewById(R.id.summWeightTxt);
        TextView summDistance = findViewById(R.id.summDistance);
        TextView summDistanceTxt = findViewById(R.id.summDistanceTxt);
        TextView summSpeed = findViewById(R.id.summSpeed);
        TextView summSpeedTxt = findViewById(R.id.summSpeedTxt);

        summDate.setText(Utils.formatDate(session.getStart(), "dd/MM/yyyy"));
        summTime.setText(Utils.formatTime(session.getStart(), session.getFinish()));

        if (session.getIdSport() == Utils.SPORT_WORKOUT)
        {
            summDistance.setVisibility(View.GONE);
            summDistanceTxt.setVisibility(View.GONE);
            summSpeed.setVisibility(View.GONE);
            summSpeedTxt.setVisibility(View.GONE);

            summExercises.setText(String.valueOf(session.getLogPlans().size()));

            int restTime = 0;
            int totalSets = 0;
            double totalWeight = 0;
            for (LogPlan logPlan : session.getLogPlans())
            {
                for (LogSet logSet : logPlan.getLogSets())
                {
                    restTime += logSet.getRest();
                    totalWeight += logSet.getWeight();
                    totalSets++;
                }
            }

            long hours = TimeUnit.SECONDS.toHours(restTime);
            long mins = TimeUnit.SECONDS.toMinutes(restTime) - TimeUnit.HOURS.toMinutes(hours);
            long secs = TimeUnit.SECONDS.toSeconds(restTime) - TimeUnit.MINUTES.toSeconds(mins) - TimeUnit.HOURS.toSeconds(hours);

            String weight = Utils.formatNumber(totalWeight) + " " + settings.getWeightUnit();

            summRest.setText(String.format(Locale.ROOT, "%d:%02d:%02d", hours, mins, secs));
            summSets.setText(String.valueOf(totalSets));
            summWeight.setText(weight);
        }
        else
        {
            summExercises.setVisibility(View.GONE);
            summExercisesTxt.setVisibility(View.GONE);
            summRest.setVisibility(View.GONE);
            summRestTxt.setVisibility(View.GONE);
            summSets.setVisibility(View.GONE);
            summSetsTxt.setVisibility(View.GONE);
            summWeight.setVisibility(View.GONE);
            summWeightTxt.setVisibility(View.GONE);

            String distance = Utils.formatNumber(session.getDistance()) + " " + settings.getDistanceUnit();
            String speed = Utils.formatNumber(session.getSpeed()) + " " + settings.getDistanceUnit() + "/h";

            summDistance.setText(distance);
            summSpeed.setText(speed);
        }
    }
}
