/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack;

import android.content.DialogInterface;
import android.os.Bundle;
import android.support.design.widget.TextInputLayout;
import android.support.v7.app.AlertDialog;
import android.support.v7.widget.Toolbar;
import android.text.InputFilter;
import android.text.InputType;
import android.view.Gravity;
import android.view.View;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import java.util.Arrays;

import edu.uoc.sporttrack.database.SettingsDatabaseHelper;
import edu.uoc.sporttrack.model.Settings;

public class SettingsActivity extends BaseDrawerActivity
{
    private SettingsDatabaseHelper db;
    private TextView distance, weight, height, sets, reps, rest;
    private Settings settings;
    private String[] distanceUnits, weightUnits, heightUnits;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        // Retrieves the content view that renders the activity.
        setContentView(R.layout.settings_activity);

        // Renders the toolbar in activity
        Toolbar toolbar = renderToolbar(false, R.string.settings_title);

        // Renders the navigation drawer
        renderNavigationDrawer(toolbar);

        // Gets arrays from resources
        distanceUnits = getResources().getStringArray(R.array.distance_units);
        weightUnits = getResources().getStringArray(R.array.weight_units);
        heightUnits = getResources().getStringArray(R.array.height_units);

        // Gets settings fields
        distance = findViewById(R.id.distanceUnit);
        weight = findViewById(R.id.weightUnit);
        height = findViewById(R.id.heightUnit);
        sets = findViewById(R.id.sets);
        reps = findViewById(R.id.reps);
        rest = findViewById(R.id.rest);

        // Creates database helper and retrieves data
        db = new SettingsDatabaseHelper(this);
        settings = db.get();
        displaySettings();
    }

    /**
     * Retrieves all data and shows in activity
     */
    private void displaySettings()
    {
        if (settings != null)
        {
            distance.setText(settings.getDistanceUnit());
            height.setText(settings.getHeightUnit());
            weight.setText(settings.getWeightUnit());
            sets.setText(String.valueOf(settings.getSets()));
            reps.setText(String.valueOf(settings.getReps()));
            rest.setText(String.valueOf(settings.getRest()));
        }
        else
        {
            settings = new Settings();
        }
    }

    /**
     * Opens dialog to update distance unit
     */
    public void onDistanceClicked(View view)
    {
        int checkedItem = -1;

        if (settings.getDistanceUnit() != null)
        {
            checkedItem = Arrays.asList(distanceUnits).indexOf(settings.getDistanceUnit());
        }
        // Shows dialog to select distance unit
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(getResources().getString(R.string.settings_distance));
        builder.setSingleChoiceItems(distanceUnits, checkedItem, new DialogInterface.OnClickListener()
        {
            @Override
            public void onClick(DialogInterface dialog, int which)
            {
                if (which != -1)
                {
                    settings.setDistanceUnit(distanceUnits[which]);
                    updateSettings();
                    dialog.dismiss();
                }
            }
        });
        builder.show();
    }

    /**
     * Opens dialog to update weight unit
     */
    public void onWeightClicked(View view)
    {
        int checkedItem = -1;

        if (settings.getWeightUnit() != null)
        {
            checkedItem = Arrays.asList(weightUnits).indexOf(settings.getWeightUnit());
        }
        // Shows dialog to select weight unit
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(getResources().getString(R.string.settings_weight));
        builder.setSingleChoiceItems(weightUnits, checkedItem, new DialogInterface.OnClickListener()
        {
            @Override
            public void onClick(DialogInterface dialog, int which)
            {
                if (which != -1)
                {
                    settings.setWeightUnit(weightUnits[which]);
                    updateSettings();
                    dialog.dismiss();
                }
            }
        });
        builder.show();
    }

    /**
     * Opens dialog to update height
     */
    public void onHeightClicked(View view)
    {
        int checkedItem = -1;

        if (settings.getHeightUnit() != null)
        {
            checkedItem = Arrays.asList(heightUnits).indexOf(settings.getHeightUnit());
        }
        // Shows dialog to select height unit
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(getResources().getString(R.string.settings_height));
        builder.setSingleChoiceItems(heightUnits, checkedItem, new DialogInterface.OnClickListener()
        {
            @Override
            public void onClick(DialogInterface dialog, int which)
            {
                if (which != -1)
                {
                    settings.setHeightUnit(heightUnits[which]);
                    updateSettings();
                    dialog.dismiss();
                }
            }
        });
        builder.show();
    }

    /**
     * Opens dialog to update sets
     */
    public void onSetsClicked(View view)
    {
        // Sets initial value
        final EditText editText = new EditText(this);
        editText.setFilters(new InputFilter[]{new InputFilter.LengthFilter(2)});
        editText.setInputType(InputType.TYPE_CLASS_NUMBER);
        editText.setGravity(Gravity.CENTER_HORIZONTAL);
        editText.setText(settings != null ? String.valueOf(settings.getSets()) : null);

        final TextInputLayout layout = new TextInputLayout(this);
        layout.setCounterEnabled(true);
        layout.setCounterMaxLength(2);
        layout.setPadding((int) getResources().getDimension(R.dimen.margin_xxxlarge), 0, (int) getResources().getDimension(R.dimen.margin_xxxlarge),
                          0);
        layout.addView(editText);

        // Shows dialog to enter sets
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(getResources().getString(R.string.settings_sets));
        builder.setView(layout);
        builder.setPositiveButton(getResources().getString(R.string.action_ok), new DialogInterface.OnClickListener()
        {
            @Override
            public void onClick(DialogInterface dialog, int which)
            {
                String value = String.valueOf(editText.getText());
                if (value.trim().isEmpty())
                {
                    // If no value is informed, an error is shown
                    Toast.makeText(getApplicationContext(), getResources().getString(R.string.err_required_field), Toast.LENGTH_SHORT).show();
                }
                else
                {
                    settings.setSets(Integer.valueOf(value));
                    updateSettings();
                }
            }
        });
        builder.setNegativeButton(getResources().getString(R.string.action_cancel), null);
        builder.show();
    }

    /**
     * Opens dialog to update reps
     */
    public void onRepsClicked(View view)
    {
        // Sets initial value
        final EditText editText = new EditText(this);
        editText.setFilters(new InputFilter[]{new InputFilter.LengthFilter(2)});
        editText.setInputType(InputType.TYPE_CLASS_NUMBER);
        editText.setGravity(Gravity.CENTER_HORIZONTAL);
        editText.setText(settings != null ? String.valueOf(settings.getReps()) : null);

        final TextInputLayout layout = new TextInputLayout(this);
        layout.setCounterEnabled(true);
        layout.setCounterMaxLength(2);
        layout.setPadding((int) getResources().getDimension(R.dimen.margin_xxxlarge), 0, (int) getResources().getDimension(R.dimen.margin_xxxlarge),
                          0);
        layout.addView(editText);

        // Shows dialog to enter reps
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(getResources().getString(R.string.settings_reps));
        builder.setView(layout);
        builder.setPositiveButton(getResources().getString(R.string.action_ok), new DialogInterface.OnClickListener()
        {
            @Override
            public void onClick(DialogInterface dialog, int which)
            {
                String value = String.valueOf(editText.getText());
                if (value.trim().isEmpty())
                {
                    // If no value is informed, an error is shown
                    Toast.makeText(getApplicationContext(), getResources().getString(R.string.err_required_field), Toast.LENGTH_SHORT).show();
                }
                else
                {
                    settings.setReps(Integer.valueOf(value));
                    updateSettings();
                }
            }
        });
        builder.setNegativeButton(getResources().getString(R.string.action_cancel), null);
        builder.show();
    }

    /**
     * Opens dialog to update rest
     */
    public void onRestClicked(View view)
    {
        // Sets initial value
        final EditText editText = new EditText(this);
        editText.setFilters(new InputFilter[]{new InputFilter.LengthFilter(3)});
        editText.setInputType(InputType.TYPE_CLASS_NUMBER);
        editText.setGravity(Gravity.CENTER_HORIZONTAL);
        editText.setText(settings != null ? String.valueOf(settings.getRest()) : null);

        final TextInputLayout layout = new TextInputLayout(this);
        layout.setCounterEnabled(true);
        layout.setCounterMaxLength(3);
        layout.setPadding((int) getResources().getDimension(R.dimen.margin_xxxlarge), 0, (int) getResources().getDimension(R.dimen.margin_xxxlarge),
                          0);
        layout.addView(editText);

        // Shows dialog to enter rest time
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(getResources().getString(R.string.settings_rest));
        builder.setView(layout);
        builder.setPositiveButton(getResources().getString(R.string.action_ok), new DialogInterface.OnClickListener()
        {
            @Override
            public void onClick(DialogInterface dialog, int which)
            {
                String value = String.valueOf(editText.getText());
                if (value.trim().isEmpty())
                {
                    // If no value is informed, an error is shown
                    Toast.makeText(getApplicationContext(), getResources().getString(R.string.err_required_field), Toast.LENGTH_SHORT).show();
                }
                else
                {
                    settings.setRest(Integer.valueOf(value));
                    updateSettings();
                }
            }
        });
        builder.setNegativeButton(getResources().getString(R.string.action_cancel), null);
        builder.show();
    }

    /**
     * Updates settings into database
     */
    private void updateSettings()
    {
        // Updates values
        if (db.update(settings) > 0)
        {
            // Refreshes values in activity
            displaySettings();
        }
        else
        {
            // Shows error if update fails
            Toast.makeText(getApplicationContext(), getResources().getString(R.string.err_settings_saved), Toast.LENGTH_SHORT).show();
        }
    }
}
