/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack;

import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.TextView;

import java.util.Date;

import edu.uoc.sporttrack.database.SessionDatabaseHelper;
import edu.uoc.sporttrack.database.SettingsDatabaseHelper;
import edu.uoc.sporttrack.model.Session;
import edu.uoc.sporttrack.model.Settings;
import edu.uoc.sporttrack.utils.Utils;

public class TimedActivity extends BaseActivity
{
    private static final long TIMER_FREQUENCY = 100;

    private SessionDatabaseHelper db;
    private long sport;
    TextView timer;
    private Session session;
    private Handler handler;
    private Runnable runnable;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        // Retrieves the content view that renders the activity.
        setContentView(R.layout.timed_activity);

        // Retrieves data from intent
        if (getIntent().getBundleExtra(Utils.BUNDLE) != null)
        {
            sport = getIntent().getBundleExtra(Utils.BUNDLE).getLong(Utils.SPORT);
        }

        // Renders the toolbar in activity
        renderToolbar(true, getResources().getString(R.string.timed_title));

        // Creates database helper
        db = new SessionDatabaseHelper(this);

        startActivity();
    }

    @Override
    public void onBackPressed()
    {
        super.onBackPressed();
        finishActivity();
    }

    /**
     * Sets up the action bar.
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        // Inflates the menu, adding items to the action bar
        getMenuInflater().inflate(R.menu.toolbar_stop, menu);

        return true;
    }

    /**
     * Called whenever an item in action bar is selected.
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        switch (item.getItemId())
        {

            // If stop icon is clicked
            case R.id.action_stop:

                closeSession();
                return true;

            default:

                return super.onOptionsItemSelected(item);
        }
    }

    /**
     * Callback method to get result from other activity.
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data)
    {
        super.onActivityResult(requestCode, resultCode, data);

        switch (requestCode)
        {
            case Utils.CODE_SUMMARY:

                // Finishes activity and sends OK code to waiting activity
                Intent intent = new Intent();
                setResult(Utils.RESULT_OK, intent);
                finish();
                break;
        }
    }

    /**
     * Sets up initial values.
     */
    private void startActivity()
    {
        // Gets current settings
        Settings settings = new SettingsDatabaseHelper(this).get();

        timer = findViewById(R.id.timer);

        // Creates session
        session = new Session();
        session.setStart(new Date());
        session.setIdSport(sport);
        session.setUnit(settings.getDistanceUnit());
        long id = db.add(session);
        session.setId(id);

        // Creates sessionHandler to display current values in activity
        handler = new Handler();
        runnable = new Runnable()
        {
            @Override
            public void run()
            {
                timer.setText(Utils.formatTime(session.getStart(), new Date()));
                handler.postDelayed(this, TIMER_FREQUENCY);
            }
        };
        handler.postDelayed(runnable, TIMER_FREQUENCY);
    }

    /**
     * Asks the user if wants to stop the current session
     */
    private void closeSession()
    {

        // Creates method to be called when confirming permission request
        Runnable runnable = new Runnable()
        {
            @Override
            public void run()
            {
                // Kills handler and finishes activity
                finishActivity();

                if (session != null)
                {
                    // Starts activity and waits for response
                    Bundle bundle = new Bundle();
                    bundle.putParcelable(Utils.SESSION, session);
                    Intent intent = new Intent(getApplicationContext(), TimedDetailActivity.class);
                    intent.putExtra(Utils.BUNDLE, bundle);
                    startActivityForResult(intent, Utils.CODE_SUMMARY);
                }
            }
        };

        Utils.showDialog(this, runnable, R.string.workout_finish_title, R.string.msg_workout_finish, -1, R.string.action_yes, R.string.action_no);
    }

    /**
     * Kills handlers and finishes activity
     */
    private void finishActivity()
    {
        handler.removeCallbacks(runnable);

        if (session != null)
        {
            session.setFinish(new Date());
            db.update(session);
        }
    }
}
