/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack.adapter;

import android.content.Context;
import android.graphics.Color;
import android.support.v4.content.ContextCompat;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.RecyclerView.ViewHolder;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.TextView;

import com.daimajia.swipe.SwipeLayout;
import com.daimajia.swipe.adapters.RecyclerSwipeAdapter;
import com.daimajia.swipe.implments.SwipeItemRecyclerMangerImpl;

import java.util.List;

import edu.uoc.sporttrack.R;
import edu.uoc.sporttrack.model.Itemizable;

/**
 * Adapter to provide binding from items to views in recycler view. It extends from
 * {@link com.daimajia.swipe.adapters.RecyclerSwipeAdapter} to facilitate swiping items.
 */
public class RecyclerAdapter extends RecyclerSwipeAdapter<RecyclerAdapter.ItemHolder>
{
    private Context context;
    private final List<Itemizable> items;
    private final int itemLayout;
    private final OnItemActionListener listener;
    private final boolean readOnly;

    /**
     * Constructor.
     */
    public RecyclerAdapter(Context context, int itemLayout, List<Itemizable> items, OnItemActionListener listener, boolean readOnly)
    {
        this.context = context;
        this.items = items;
        this.itemLayout = itemLayout;
        this.listener = listener;
        this.readOnly = readOnly;
    }

    /**
     * Creates item view holder.
     * It overrides {@link android.support.v7.widget.RecyclerView.Adapter#onCreateViewHolder(ViewGroup, int)}
     */
    @Override
    public ItemHolder onCreateViewHolder(ViewGroup parent, int viewType)
    {
        View view = LayoutInflater.from(parent.getContext()).inflate(itemLayout, parent, false);
        return new ItemHolder(view);
    }

    /**
     * Binds item data to view holder at the specified position. It overrides
     * {@link android.support.v7.widget.RecyclerView.Adapter#onBindViewHolder(ViewHolder, int)}
     */
    @Override
    public void onBindViewHolder(final ItemHolder holder, final int position)
    {
        // Binding item to holder
        final Itemizable item = items.get(position);
        holder.item = item;

        // Sets image from resources
        if (holder.image != null)
        {
            holder.image.setImageResource(item.getImage());
        }

        // Sets and highlights title
        if (holder.title != null)
        {
            holder.title.setText(item.getItemTitle());
        }

        // Sets subtitle
        if (holder.subtitle != null)
        {
            holder.subtitle.setText(item.getItemSubtitle());
        }

        // Sets remark
        if (holder.remark != null)
        {
            holder.remark.setText(item.getItemRemark());
        }

        // Sets sets
        if (holder.sets != null)
        {
            holder.sets.setText(item.getItemSets());
        }

        // Sets repetitions
        if (holder.reps != null)
        {
            holder.reps.setText(item.getItemReps());
        }

        // Sets rest time
        if (holder.rest != null)
        {
            holder.rest.setText(item.getItemRest());
        }

        // Sets weight
        if (holder.weight != null)
        {
            holder.weight.setText(item.getItemWeight());
        }

        // Configures editable, deletable, selectable and highlighted items
        applyEditableMode(holder, item);
        applyDeletableMode(holder, item);
        applyHighlightedMode(holder, item);
        applySelectableMode(holder, item);

        // Configures swipe layout
        applySwipeMode(holder, item, position);

        // Configures draggable items
        applyDraggableMode(holder);
    }

    /**
     * Returns the number of items. It overrides {@link RecyclerView.Adapter#getItemCount()}
     */
    @Override
    public int getItemCount()
    {
        return items.size();
    }

    /**
     * Returns resource id of the item being swiped. It overrides
     * {@link com.daimajia.swipe.adapters.RecyclerSwipeAdapter#getSwipeLayoutResourceId(int)}
     */
    @Override
    public int getSwipeLayoutResourceId(int position)
    {
        return R.id.item_row;
    }

    /**
     * Returns the helper class to help the recycler view to manage swiped items. See {@link com.daimajia.swipe.implments.SwipeItemRecyclerMangerImpl}
     */
    public SwipeItemRecyclerMangerImpl getManager()
    {
        return mItemManger;
    }

    /**
     * Configures swipe layout
     */
    private void applySwipeMode(ItemHolder holder, final Itemizable item, int position)
    {
        boolean swipeable = (item.isEditable() || item.isDeletable()) && !readOnly;

        if (!readOnly && holder.layout != null && holder.layout instanceof SwipeLayout)
        {
            ((SwipeLayout) holder.layout).setRightSwipeEnabled(false);
            ((SwipeLayout) holder.layout).setLeftSwipeEnabled(swipeable);
            ((SwipeLayout) holder.layout).setShowMode(SwipeLayout.ShowMode.PullOut);
            ((SwipeLayout) holder.layout).addDrag(SwipeLayout.DragEdge.Left, holder.layout.findViewById(R.id.background_view));
            holder.background.setVisibility(swipeable ? View.VISIBLE : View.INVISIBLE);

            if (!item.isHighlighted())
            {
                holder.foreground.setBackgroundColor(swipeable ? Color.WHITE : ContextCompat.getColor(context, R.color.softGray));
            }

            // Bins swipe helper to view
            mItemManger.bindView(holder.itemView, position);
        }
    }

    /**
     * Configures highlight mode
     */
    private void applyHighlightedMode(ItemHolder holder, final Itemizable item)
    {
        if (holder.foreground != null)
        {
            holder.foreground.setBackgroundColor(item.isHighlighted() ? ContextCompat.getColor(context, R.color.green) : Color.TRANSPARENT);
        }

        if (holder.title != null)
        {
            holder.title.setTextColor(item.isHighlighted() ? Color.WHITE : ContextCompat.getColor(context, R.color.softBlack));
        }

        if (holder.subtitle != null)
        {
            holder.subtitle.setTextColor(item.isHighlighted() ? Color.WHITE : ContextCompat.getColor(context, R.color.gray));
        }

        if (holder.sets != null)
        {
            holder.sets.setTextColor(item.isHighlighted() ? Color.WHITE : ContextCompat.getColor(context, R.color.softBlack));
        }

        if (holder.setsTitle != null)
        {
            holder.setsTitle.setTextColor(item.isHighlighted() ? Color.WHITE : ContextCompat.getColor(context, R.color.softBlack));
        }

        if (holder.reps != null)
        {
            holder.reps.setTextColor(item.isHighlighted() ? Color.WHITE : ContextCompat.getColor(context, R.color.softBlack));
        }

        if (holder.repsTitle != null)
        {
            holder.repsTitle.setTextColor(item.isHighlighted() ? Color.WHITE : ContextCompat.getColor(context, R.color.softBlack));
        }

        if (holder.rest != null)
        {
            holder.rest.setTextColor(item.isHighlighted() ? Color.WHITE : ContextCompat.getColor(context, R.color.softBlack));
        }

        if (holder.restTitle != null)
        {
            holder.restTitle.setTextColor(item.isHighlighted() ? Color.WHITE : ContextCompat.getColor(context, R.color.softBlack));
        }
    }

    /**
     * Configures draggable mode
     */
    private void applyDraggableMode(final ItemHolder holder)
    {
        if (holder.handle != null)
        {
            holder.handle.setOnTouchListener(new View.OnTouchListener()
            {
                @Override
                public boolean onTouch(View v, MotionEvent event)
                {
                    if (event.getActionMasked() == MotionEvent.ACTION_DOWN)
                    {
                        listener.onItemDrag(holder);
                    }
                    return false;
                }
            });
        }
    }

    /**
     * Configures editable mode
     */
    private void applyEditableMode(ItemHolder holder, final Itemizable item)
    {
        if (!readOnly)
        {
            // Configures row to be edited
            if (item.isEditable())
            {
                holder.editBtn.setVisibility(View.VISIBLE);
                holder.editBtn.setOnClickListener(new View.OnClickListener()
                {
                    @Override
                    public void onClick(View v)
                    {
                        ((OnItemActionListener) context).onItemEdit(item);
                    }
                });
            }
            else if (holder.editBtn != null)
            {
                // Removes edit button if item is not editable
                holder.editBtn.setVisibility(View.INVISIBLE);
            }
        }
    }

    /**
     * Configures deletable mode
     */
    private void applyDeletableMode(ItemHolder holder, final Itemizable item)
    {
        if (!readOnly)
        {
            // Configures row to be deleted
            if (item.isDeletable())
            {
                holder.deleteBtn.setVisibility(View.VISIBLE);
                holder.deleteBtn.setOnClickListener(new View.OnClickListener()
                {
                    @Override
                    public void onClick(View v)
                    {
                        ((OnItemActionListener) context).onItemDelete(item);
                    }
                });
            }
            else if (holder.deleteBtn != null)
            {
                // Removes edit button if item is not deletable
                holder.deleteBtn.setVisibility(View.INVISIBLE);
            }
        }
    }

    /**
     * Configures selectable mode
     */
    private void applySelectableMode(ItemHolder holder, final Itemizable item)
    {
        if (item.isSelectable())
        {
            if (holder.layout instanceof SwipeLayout)
            {
                ((SwipeLayout) holder.layout).getSurfaceView().setOnClickListener(new View.OnClickListener()
                {
                    @Override
                    public void onClick(View v)
                    {
                        listener.onItemSelect(item);
                    }
                });
            }
            else if (holder.layout != null)
            {
                holder.layout.setOnClickListener(new View.OnClickListener()
                {
                    @Override
                    public void onClick(View v)
                    {
                        listener.onItemSelect(item);
                    }
                });
            }
        }
        else if (!item.isHighlighted())
        {
            holder.foreground.setBackgroundColor(ContextCompat.getColor(context, R.color.softGray));
        }
    }

    /**
     * View holder to provide binding to items in recycler view.
     */
    public class ItemHolder extends ViewHolder
    {
        Itemizable item;
        View layout, foreground, background, handle;
        TextView title, subtitle, remark, sets, reps, rest, weight, setsTitle, repsTitle, restTitle;
        ImageButton editBtn, deleteBtn;
        ImageView image;

        /**
         * Constructor.
         */
        ItemHolder(View view)
        {
            super(view);

            layout = view.findViewById(R.id.item_row);
            foreground = view.findViewById(R.id.foreground_view);
            background = view.findViewById(R.id.background_view);
            handle = view.findViewById(R.id.handle);

            title = view.findViewById(R.id.item_title);
            subtitle = view.findViewById(R.id.item_subtitle);
            remark = view.findViewById(R.id.item_remark);
            sets = view.findViewById(R.id.item_sets);
            reps = view.findViewById(R.id.item_reps);
            rest = view.findViewById(R.id.item_rest);
            weight = view.findViewById(R.id.item_weight);
            setsTitle = view.findViewById(R.id.item_sets_title);
            repsTitle = view.findViewById(R.id.item_reps_title);
            restTitle = view.findViewById(R.id.item_rest_title);

            deleteBtn = view.findViewById(R.id.delete_button);
            editBtn = view.findViewById(R.id.edit_button);
            image = view.findViewById(R.id.item_image);
        }
    }

}
