/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack.database;

import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

public class DatabaseHelper extends SQLiteOpenHelper
{
    // Database Version
    private static final int DATABASE_VERSION = 1;

    private static final String DATABASE_PATH = "/data/data/edu.uoc.sporttrack/databases/";
    private static final String DATABASE_NAME = "SportTrack.db";

    // Tables
    static final String TAB_BODY = "bodyparts";
    static final String TAB_DAY = "days";
    static final String TAB_EXERC = "exercises";
    static final String TAB_LOGPLAN = "log_plans";
    static final String TAB_LOGSET = "log_sets";
    static final String TAB_POINT = "points";
    static final String TAB_PLAN = "plans";
    static final String TAB_PROFILE = "profile";
    static final String TAB_ROUTINE = "routines";
    static final String TAB_SESSION = "sessions";
    static final String TAB_SETTS = "settings";

    // Columns
    static final String COL_BODYPART = "bodypart";
    static final String COL_BUILTIN = "built_in";
    static final String COL_CAL = "calories";
    static final String COL_CURRENT = "current";
    static final String COL_DATE = "log_date";
    static final String COL_DAY = "day";
    static final String COL_DAYNAME = "day_name";
    static final String COL_DESC = "description";
    static final String COL_DIST = "distance";
    static final String COL_DOB = "dob";
    static final String COL_EXERC = "exercise";
    static final String COL_FINISH = "finish";
    static final String COL_GENDER = "gender";
    static final String COL_HEIGHT = "height";
    static final String COL_ID = "_id";
    static final String COL_IDBODYPART = "id_bodypart";
    static final String COL_IDDAY = "id_day";
    static final String COL_IDEXERC = "id_exercise";
    static final String COL_IDLOG = "id_log";
    static final String COL_IDPLAN = "id_plan";
    static final String COL_IDROUTINE = "id_routine";
    static final String COL_IDSESSION = "id_session";
    static final String COL_IDSPORT = "id_sport";
    static final String COL_LAT = "latitude";
    static final String COL_LONG = "longitude";
    static final String COL_NAME = "name";
    static final String COL_POS = "position";
    static final String COL_REPS = "reps";
    static final String COL_REST = "rest";
    static final String COL_SPEED = "speed";
    static final String COL_START = "start";
    static final String COL_SETS = "sets";
    static final String COL_UNIT = "unit";
    static final String COL_WEIGHT = "weight";

    private final Context context;

    /**
     * Takes and keeps a reference of the passed context in order to access to the application
     * assets and resources.
     */
    public DatabaseHelper(Context context)
    {
        super(context, DATABASE_NAME, null, DATABASE_VERSION);
        this.context = context;
    }

    /**
     * Called first time when the database is created
     */
    @Override
    public void onCreate(SQLiteDatabase db)
    {
        // It does nothing since the database it will be copied from assets folder
    }

    @Override
    public void onOpen(SQLiteDatabase db)
    {
        super.onOpen(db);
        String query = String.format("PRAGMA FOREIGN_KEYS=%s", "ON");
        db.execSQL(query);
    }

    /**
     * Called when upgrade is done
     */
    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion)
    {

    }

    /**
     * Creates a empty database on the device and rewrites it with database from the assets folder.
     */
    public void createDatabase() throws Exception
    {
        // Checks if the database doesn't exist and copies it from the assets folder
        if (!checkDatabase())
        {
            // Creates an empty database on the device
            this.getReadableDatabase();

            try
            {
                // Copies database in assets folder to the empty database just created
                copyDataBase();
            }
            catch (IOException e)
            {
                throw new Exception("Error copying database from assets folder");
            }
        }
    }

    /**
     * Checks if the database already exists in path
     */
    private boolean checkDatabase()
    {
        return context.getDatabasePath(DATABASE_NAME).exists();
    }

    /**
     * Copies database from assets folder to the just created empty database in the system folder
     */
    private void copyDataBase() throws IOException
    {
        // Open local database from assets folder as the input stream
        InputStream inputStream = context.getAssets().open(DATABASE_NAME);

        // Database file that have been just created
        String databaseFile = DATABASE_PATH + DATABASE_NAME;

        // Open the empty databse as the output stream
        OutputStream outputStream = new FileOutputStream(databaseFile);

        // Transfer bytes from the input file to the output file
        byte[] buffer = new byte[1024];
        int length;
        while ((length = inputStream.read(buffer)) > 0)
        {
            outputStream.write(buffer, 0, length);
        }

        // Close all streams
        outputStream.flush();
        outputStream.close();
        inputStream.close();
    }

}
