/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack.database;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.sporttrack.model.Day;
import edu.uoc.sporttrack.utils.Utils;

public class DayDatabaseHelper extends DatabaseHelper
{
    private Context context;

    /**
     * Constructor
     */
    public DayDatabaseHelper(Context context)
    {
        super(context);
        this.context = context;
    }

    /**
     * Adds item to database
     */
    private long add(Day day)
    {
        // Opens database in write mode
        SQLiteDatabase db = this.getWritableDatabase();

        ContentValues values = new ContentValues();
        values.put(COL_DAY, day.getPosition());
        values.put(COL_NAME, day.getName());
        values.put(COL_IDROUTINE, day.getIdRoutine());

        // Inserting row
        long result = db.insert(TAB_DAY, null, values);

        // Closing database connection
        db.close();

        return result;
    }

    /**
     * Gets item from database by id
     */
    public Day get(long id)
    {
        Day day = null;

        // Opens database in read mode
        SQLiteDatabase db = this.getReadableDatabase();

        if (db == null)
        {
            return null;
        }

        String[] cols = new String[]{COL_ID, COL_DAY, COL_NAME, COL_IDROUTINE};
        String[] args = new String[]{String.valueOf(id)};
        String where = COL_ID + "=?";

        Cursor cursor = db.query(TAB_DAY, cols, where, args, null, null, null, null);

        // We ensure there is at least a record
        if (cursor.moveToNext())
        {
            day = new Day();
            day.setId(cursor.getLong(0));
            day.setPosition(cursor.getInt(1));
            day.setName(cursor.getString(2));
            day.setIdRoutine(cursor.getLong(3));
            day.setContext(context);
        }

        // Closing cursor and database connection
        cursor.close();
        db.close();

        return day;
    }

    /**
     * Updates item into database
     */
    private long update(Day day)
    {
        long result = 0;

        // Opens database in write mode
        SQLiteDatabase db = this.getWritableDatabase();

        if (db == null)
        {
            return result;
        }

        ContentValues values = new ContentValues();
        values.put(COL_DAY, day.getPosition());
        values.put(COL_NAME, day.getName());
        values.put(COL_IDROUTINE, day.getIdRoutine());

        String[] args = new String[]{String.valueOf(day.getId())};
        String where = COL_ID + "=?";

        // Updating row
        result = db.update(TAB_DAY, values, where, args);

        // Closing database connection
        db.close();

        return result;
    }

    /**
     * Deletes item from database
     */
    public long delete(long id)
    {
        // Opens database in write mode
        SQLiteDatabase db = this.getWritableDatabase();

        long result = 0;

        if (db == null)
        {
            return result;
        }

        String[] args = new String[]{String.valueOf(id)};
        String where = COL_ID + "=?";

        // Deleting row
        result = db.delete(TAB_DAY, where, args);

        // Closing database connection
        db.close();

        return result;
    }

    /**
     * Gets all items from database by routine
     */
    public List<Day> getAll(long idRoutine)
    {
        // Opens database in read mode
        SQLiteDatabase db = this.getReadableDatabase();

        if (db == null)
        {
            return null;
        }

        String cols = "SELECT d._id, d.day, d.name, d.id_routine, max(s.start), ";
        String subquery = "(SELECT count(p._id) FROM plans p WHERE p.id_day = d._id) numPlans  ";
        String tables = "FROM days d LEFT JOIN sessions s ON (d._id = s.id_day) ";
        String where = "WHERE d.id_routine = ? ";
        String groupBy = "GROUP BY d._id, d.day, d.name, d.id_routine ";
        String orderBy = "ORDER BY d.day ASC";
        String[] args = {String.valueOf(idRoutine)};

        Cursor cursor = db.rawQuery(cols + subquery + tables + where + groupBy + orderBy, args);

        List<Day> days = new ArrayList<>();

        while (cursor.moveToNext())
        {
            Day day = new Day();
            day.setId(cursor.getLong(0));
            day.setPosition(cursor.getInt(1));
            day.setName(cursor.getString(2));
            day.setIdRoutine(cursor.getLong(3));
            day.setLastLogged(Utils.getDate(cursor.getString(4)));
            day.setNumPlans(cursor.getInt(5));
            day.setContext(context);
            days.add(day);
        }

        // Closing cursor and database connection
        cursor.close();
        db.close();

        // return routineDay
        return days;
    }

    /**
     * Adds or updates an item depending if exists
     */
    public long saveOrUpdate(Day day)
    {
        if (day.getId() > 0)
        {
            return update(day);
        }

        return add(day);
    }
}