/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack.database;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.sporttrack.R;
import edu.uoc.sporttrack.model.Exercise;

public class ExerciseDatabaseHelper extends DatabaseHelper
{
    private boolean selectionMode;
    private Context context;

    /**
     * Constructor
     */
    public ExerciseDatabaseHelper(Context context, boolean selectionMode)
    {
        super(context);
        this.context = context;
        this.selectionMode = selectionMode;
    }

    /**
     * Adds item to database
     */
    private long add(Exercise exercise)
    {
        // Opens database in write mode
        SQLiteDatabase db = this.getWritableDatabase();

        ContentValues values = new ContentValues();
        values.put(COL_IDBODYPART, exercise.getIdBodypart());
        values.put(COL_BUILTIN, 0);
        values.put(COL_NAME, exercise.getName());
        values.put(COL_DESC, exercise.getDescription());

        // Inserting row
        long result = db.insert(TAB_EXERC, null, values);

        // Closing database connection
        db.close();

        return result;
    }

    /**
     * Gets item from database by id
     */
    Exercise get(long id)
    {
        Exercise exercise = null;

        // Opens database in read mode
        SQLiteDatabase db = this.getReadableDatabase();

        if (db == null)
        {
            return null;
        }

        String[] cols = new String[]{COL_ID, COL_IDBODYPART, COL_BUILTIN, COL_NAME, COL_DESC};
        String[] args = new String[]{String.valueOf(id)};
        String where = COL_ID + "=?";

        Cursor cursor = db.query(TAB_EXERC, cols, where, args, null, null, null, null);

        // We ensure there is at least a record
        if (cursor.moveToNext())
        {
            exercise = new Exercise();
            exercise.setId(cursor.getLong(0));
            exercise.setIdBodypart(cursor.getLong(1));
            exercise.setBuiltIn(cursor.getInt(2) == 1);
            exercise.setName(cursor.getString(3));
            exercise.setDescription(cursor.getString(4));
            exercise.setSelectionMode(selectionMode);
            exercise.setType(cursor.getInt(2) == 1 ? context.getString(R.string.exercise_built_in) : context.getString(R.string.exercise_custom));
        }

        cursor.close();

        // Closing database connection
        db.close();

        // return exercise
        return exercise;
    }

    /**
     * Updates item into database
     */
    private long update(Exercise exercise)
    {
        // Opens database in write mode
        SQLiteDatabase db = this.getWritableDatabase();

        long result = 0;

        if (db == null)
        {
            return result;
        }

        ContentValues values = new ContentValues();
        values.put(COL_NAME, exercise.getName());
        values.put(COL_DESC, exercise.getDescription());
        values.put(COL_IDBODYPART, exercise.getIdBodypart());

        String[] args = new String[]{String.valueOf(exercise.getId())};
        String where = COL_ID + "=?";

        // Updating row
        result = db.update(TAB_EXERC, values, where, args);

        // Closing database connection
        db.close();

        return result;
    }

    /**
     * Deletes item from database
     */
    public long delete(long id)
    {
        // Opens database in write mode
        SQLiteDatabase db = this.getWritableDatabase();

        long result = 0;

        if (db == null)
        {
            return result;
        }

        String[] args = new String[]{String.valueOf(id)};
        String where = COL_ID + "=?";

        // Deleting row
        result = db.delete(TAB_EXERC, where, args);

        // Closing database connection
        db.close();

        return result;
    }

    /**
     * Gets all items from database by bodypart or name
     */
    public List<Exercise> getAll(long idBodyPart, String name)
    {
        // Opens database in read mode
        SQLiteDatabase db = this.getReadableDatabase();

        if (db == null)
        {
            return null;
        }

        String[] cols = new String[]{COL_ID, COL_IDBODYPART, COL_BUILTIN, COL_NAME, COL_DESC};

        ArrayList<String> args = new ArrayList<>();
        String where = "";

        if (idBodyPart > 0)
        {
            args.add(String.valueOf(idBodyPart));
            where += COL_IDBODYPART + "=?";
        }

        if (name != null && !name.isEmpty())
        {
            args.add("%" + name.toUpperCase() + "%");
            if (!where.isEmpty())
            {

                where += " AND ";
            }
            where += "UPPER(" + COL_NAME + ") LIKE ?";
        }

        String orderBy = COL_NAME + " ASC";

        Cursor cursor = db.query(TAB_EXERC, cols, where, args.toArray(new String[0]), null, null, orderBy, null);

        List<Exercise> exercises = new ArrayList<>();

        while (cursor.moveToNext())
        {
            Exercise exercise = new Exercise();
            exercise.setId(cursor.getLong(0));
            exercise.setIdBodypart(cursor.getLong(1));
            exercise.setBuiltIn(cursor.getInt(2) == 1);
            exercise.setName(cursor.getString(3));
            exercise.setDescription(cursor.getString(4));
            exercise.setSelectionMode(selectionMode);
            exercise.setType(cursor.getInt(2) == 1 ? context.getString(R.string.exercise_built_in) : context.getString(R.string.exercise_custom));
            exercises.add(exercise);
        }

        cursor.close();

        // Closing database connection
        db.close();

        // return exercise
        return exercises;
    }

    /**
     * Adds or updates an item depending if exists
     */
    public long saveOrUpdate(Exercise exercise)
    {
        if (exercise.getId() > 0)
        {
            return update(exercise);
        }

        return add(exercise);
    }
}
