/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack.database;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.sporttrack.model.Plan;

public class PlanDatabaseHelper extends DatabaseHelper
{
    /**
     * Constructor
     */
    public PlanDatabaseHelper(Context context)
    {
        super(context);
    }

    /**
     * Adds item to database
     */
    public long add(Plan plan)
    {
        long result;

        // Opens database in write mode
        SQLiteDatabase db = this.getWritableDatabase();

        ContentValues values = new ContentValues();
        values.put(COL_IDDAY, plan.getIdDay());
        values.put(COL_IDEXERC, plan.getIdExercise());
        values.put(COL_POS, plan.getPosition());
        values.put(COL_SETS, plan.getSets());
        values.put(COL_REPS, plan.getReps());
        values.put(COL_REST, plan.getRest());

        // Inserting row
        result = db.insert(TAB_PLAN, null, values);

        // Closing database connection
        db.close();

        return result;
    }

    /**
     * Gets item from database by id
     */
    public Plan get(long id)
    {
        Plan plan = null;

        // Opens database in read mode
        SQLiteDatabase db = this.getReadableDatabase();

        if (db == null)
        {
            return null;
        }

        String[] cols = new String[]{COL_ID, COL_IDDAY, COL_IDEXERC, COL_POS, COL_SETS, COL_REPS, COL_REST};
        String[] args = new String[]{String.valueOf(id)};
        String where = COL_ID + "=?";

        Cursor cursor = db.query(TAB_PLAN, cols, where, args, null, null, null, null);

        // We ensure there is at least a record
        if (cursor.moveToNext())
        {
            plan = new Plan();
            plan.setId(cursor.getLong(0));
            plan.setIdDay(cursor.getLong(1));
            plan.setIdExercise(cursor.getLong(2));
            plan.setPosition(cursor.getInt(3));
            plan.setSets(cursor.getInt(4));
            plan.setReps(cursor.getInt(5));
            plan.setRest(cursor.getInt(6));
        }

        // Closing cursor and database connection
        cursor.close();
        db.close();

        return plan;
    }

    /**
     * Updates item into database
     */
    public long update(Plan plan)
    {
        long result = 0;

        // Opens database in write mode
        SQLiteDatabase db = this.getWritableDatabase();

        if (db == null)
        {
            return result;
        }

        ContentValues values = new ContentValues();
        values.put(COL_IDDAY, plan.getIdDay());
        values.put(COL_IDEXERC, plan.getIdExercise());
        values.put(COL_POS, plan.getPosition());
        values.put(COL_SETS, plan.getSets());
        values.put(COL_REPS, plan.getReps());
        values.put(COL_REST, plan.getRest());

        String[] args = new String[]{String.valueOf(plan.getId())};
        String where = COL_ID + "=?";

        // Updating row
        result = db.update(TAB_PLAN, values, where, args);

        // Closing database connection
        db.close();

        return result;
    }

    /**
     * Updates a list of items into database
     */
    public void update(List<Plan> plans)
    {
        // Opens database in write mode
        SQLiteDatabase db = this.getWritableDatabase();

        if (db == null)
        {
            return;
        }

        String where = COL_ID + "=?";

        for (Plan plan : plans)
        {
            ContentValues values = new ContentValues();
            values.put(COL_IDDAY, plan.getIdDay());
            values.put(COL_IDEXERC, plan.getIdExercise());
            values.put(COL_POS, plan.getPosition());
            values.put(COL_SETS, plan.getSets());
            values.put(COL_REPS, plan.getReps());
            values.put(COL_REST, plan.getRest());

            String[] args = new String[]{String.valueOf(plan.getId())};

            // Updating row
            db.update(TAB_PLAN, values, where, args);
        }

        // Closing database connection
        db.close();
    }

    /**
     * Deletes item from database
     */
    public long delete(long id)
    {
        // Opens database in write mode
        SQLiteDatabase db = this.getWritableDatabase();

        long result = 0;

        if (db == null)
        {
            return result;
        }

        String[] args = new String[]{String.valueOf(id)};
        String where = COL_ID + "=?";

        // Deleting row
        result = db.delete(TAB_PLAN, where, args);

        // Closing database connection
        db.close();

        return result;
    }

    /**
     * Gets all items from database by day
     */
    public List<Plan> getAll(long idDay)
    {
        // Opens database in read mode
        SQLiteDatabase db = this.getReadableDatabase();

        if (db == null)
        {
            return null;
        }

        String tables = TAB_PLAN + " d , " + TAB_EXERC + " e, " + TAB_BODY + " b";
        String[] cols = new String[]{"d." + COL_ID, "d." + COL_IDDAY, "d." + COL_IDEXERC, "d." + COL_POS, "d." + COL_SETS, "d." + COL_REPS,
                                     "d." + COL_REST, "e." + COL_NAME, "b." + COL_NAME};
        String where = "d." + COL_IDEXERC + " = " + "e." + COL_ID + " AND " + "d." + COL_IDDAY + "" + " = ? AND " + "b." + COL_ID + "" + " = e." +
                COL_IDBODYPART;
        String[] args = {String.valueOf(idDay),};
        String orderBy = "d." + COL_POS + " ASC";

        Cursor cursor = db.query(tables, cols, where, args, null, null, orderBy, null);

        List<Plan> plans = new ArrayList<>();

        while (cursor.moveToNext())
        {
            Plan plan = new Plan();
            plan.setId(cursor.getLong(0));
            plan.setIdDay(cursor.getLong(1));
            plan.setIdExercise(cursor.getLong(2));
            plan.setPosition(cursor.getInt(3));
            plan.setSets(cursor.getInt(4));
            plan.setReps(cursor.getInt(5));
            plan.setRest(cursor.getInt(6));
            plan.setExerciseName(cursor.getString(7));
            plan.setBodypart(cursor.getString(8));
            plans.add(plan);
        }

        // Closing cursor and database connection
        cursor.close();
        db.close();

        // return routineDayExercise
        return plans;
    }

    /**
     * Gets next order value
     */
    public int nextPosition(long idDay)
    {
        int max = 0;

        // Opens database in read mode
        SQLiteDatabase db = this.getReadableDatabase();

        if (db == null)
        {
            return max + 1;
        }

        String[] cols = new String[]{"MAX(" + COL_POS + ")"};
        String[] args = new String[]{String.valueOf(idDay)};
        String where = COL_IDDAY + "=?";

        Cursor cursor = db.query(TAB_PLAN, cols, where, args, null, null, null, null);
        if (cursor.moveToNext())
        {
            max = cursor.getInt(0);
        }

        // Closing cursor and database connection
        cursor.close();
        db.close();

        return max + 1;
    }
}