/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack.database;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.sporttrack.model.Routine;

public class RoutineDatabaseHelper extends DatabaseHelper
{
    private Context context;

    /**
     * Constructor
     */
    public RoutineDatabaseHelper(Context context)
    {
        super(context);
        this.context = context;
    }

    /**
     * Adds item to database
     */
    private long add(Routine routine)
    {
        // Opens database in write mode
        SQLiteDatabase db = this.getWritableDatabase();

        // Clearing all default routines
        if (routine.isCurrent())
        {
            ContentValues values = new ContentValues();
            values.put(COL_CURRENT, 0);
            String where = COL_CURRENT + "=?";
            String[] args = new String[]{"1"};

            db.update(TAB_ROUTINE, values, where, args);
        }

        ContentValues values = new ContentValues();
        values.put(COL_CURRENT, 0);
        values.put(COL_NAME, routine.getName());
        values.put(COL_DESC, routine.getDescription());
        values.put(COL_CURRENT, routine.isCurrent() ? 1 : 0);

        // Inserting row
        long result = db.insert(TAB_ROUTINE, null, values);

        // Closing database connection
        db.close();

        return result;
    }

    /**
     * Gets item from database by id
     */
    public Routine get(long id)
    {
        Routine routine = null;

        // Opens database in read mode
        SQLiteDatabase db = this.getReadableDatabase();

        if (db == null)
        {
            return null;
        }

        String[] cols = new String[]{COL_ID, COL_NAME, COL_DESC, COL_CURRENT};
        String[] args = new String[]{String.valueOf(id)};
        String where = COL_ID + "=?";

        Cursor cursor = db.query(TAB_ROUTINE, cols, where, args, null, null, null, null);

        // We ensure there is at least a record
        if (cursor.moveToNext())
        {
            routine = new Routine();
            routine.setId(cursor.getLong(0));
            routine.setName(cursor.getString(1));
            routine.setDescription(cursor.getString(2));
            routine.setCurrent(cursor.getInt(3) == 1);
            routine.setContext(context);
        }

        // Closing cursor and database connection
        cursor.close();
        db.close();

        return routine;
    }

    /**
     * Updates item into database
     */
    private long update(Routine routine)
    {
        long result = 0;

        // Opens database in write mode
        SQLiteDatabase db = this.getWritableDatabase();

        if (db == null)
        {
            return result;
        }

        // Clearing all default routines
        if (routine.isCurrent())
        {
            ContentValues values = new ContentValues();
            values.put(COL_CURRENT, 0);
            String where = COL_CURRENT + "=?";
            String[] args = new String[]{"1"};

            db.update(TAB_ROUTINE, values, where, args);
        }

        ContentValues values = new ContentValues();
        values.put(COL_NAME, routine.getName());
        values.put(COL_DESC, routine.getDescription());
        values.put(COL_CURRENT, routine.isCurrent() ? 1 : 0);

        String[] args = new String[]{String.valueOf(routine.getId())};
        String where = COL_ID + "=?";

        // Updating row
        result = db.update(TAB_ROUTINE, values, where, args);

        // Closing database connection
        db.close();

        return result;
    }

    /**
     * Deletes item from database
     */
    public long delete(long id)
    {
        // Opens database in write mode
        SQLiteDatabase db = this.getWritableDatabase();

        long result = 0;

        if (db == null)
        {
            return result;
        }

        String[] args = new String[]{String.valueOf(id)};
        String where = COL_ID + "=?";

        // Deleting row
        result = db.delete(TAB_ROUTINE, where, args);

        // Closing database connection
        db.close();

        return result;
    }

    /**
     * Gets all items from database by name
     */
    public List<Routine> getAll(String name)
    {
        // Opens database in read mode
        SQLiteDatabase db = this.getReadableDatabase();

        if (db == null)
        {
            return null;
        }

        String cols = "SELECT r._id, r.name, r.description, r.current, count(d._id) ";
        String tables = "FROM routines r left join days d on (r._id = d.id_routine) ";
        String where = "";
        String groupBy = "GROUP BY r._id, r._id, r.name, r.description, r.current";
        String[] args = null;

        if (name != null && !name.isEmpty())
        {
            where = "WHERE upper(r.name) LIKE ? ";
            args = new String[]{"%" + name.toUpperCase() + "%"};
        }

        Cursor cursor = db.rawQuery(cols + tables + where + groupBy, args);

        List<Routine> routines = new ArrayList<>();

        while (cursor.moveToNext())
        {
            Routine routine = new Routine();
            routine.setId(cursor.getInt(0));
            routine.setName(cursor.getString(1));
            routine.setDescription(cursor.getString(2));
            routine.setCurrent(cursor.getInt(3) == 1);
            routine.setNumDays(cursor.getInt(4));
            routine.setContext(context);
            routines.add(routine);
        }

        // Closing cursor and database connection
        cursor.close();
        db.close();

        // return routine
        return routines;
    }

    /**
     * Adds or updates an item depending if exists
     */
    public long saveOrUpdate(Routine routine)
    {
        if (routine.getId() > 0)
        {
            return update(routine);
        }

        return add(routine);
    }

    /**
     * Gets current routine
     */
    public Routine getCurrent()
    {
        Routine routine = null;

        // Opens database in read mode
        SQLiteDatabase db = this.getReadableDatabase();

        if (db == null)
        {
            return null;
        }

        String[] cols = new String[]{COL_ID, COL_NAME, COL_DESC, COL_CURRENT};
        String[] args = new String[]{String.valueOf(1)};
        String where = COL_CURRENT + "=?";

        Cursor cursor = db.query(TAB_ROUTINE, cols, where, args, null, null, null, null);

        // We ensure there is at least a record
        if (cursor.moveToNext())
        {
            routine = new Routine();
            routine.setId(cursor.getLong(0));
            routine.setName(cursor.getString(1));
            routine.setDescription(cursor.getString(2));
            routine.setCurrent(cursor.getInt(3) == 1);
        }

        // Closing cursor and database connection
        cursor.close();
        db.close();

        return routine;
    }
}