/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack.database;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import edu.uoc.sporttrack.model.LogPlan;
import edu.uoc.sporttrack.model.LogSet;
import edu.uoc.sporttrack.model.Point;
import edu.uoc.sporttrack.model.Session;
import edu.uoc.sporttrack.model.WorkoutDay;
import edu.uoc.sporttrack.utils.Utils;

public class SessionDatabaseHelper extends DatabaseHelper
{
    private Context context;

    /**
     * Constructor
     */
    public SessionDatabaseHelper(Context context)
    {
        super(context);
        this.context = context;
    }

    /**
     * Gets current workout session
     */
    public Session getActiveWorkoutSession()
    {
        Session session = null;

        // Opens database in read mode
        SQLiteDatabase db = this.getReadableDatabase();

        if (db == null)
        {
            return null;
        }

        String[] cols = new String[]{COL_ID, COL_IDSPORT, COL_START, COL_FINISH, COL_IDDAY, COL_DAYNAME};
        String where = COL_START + " > strftime('%Y-%m-%d', 'now') AND " + COL_FINISH + " IS NULL AND " + COL_IDSPORT + " = ?";
        String[] args = new String[]{String.valueOf(Utils.SPORT_WORKOUT)};

        Cursor cursor = db.query(TAB_SESSION, cols, where, args, null, null, null, null);

        if (cursor.moveToFirst())
        {
            session = new Session();
            session.setId(cursor.getLong(0));
            session.setIdSport(cursor.getLong(1));
            session.setStart(Utils.getDate(cursor.getString(2)));
            session.setFinish(Utils.getDate(cursor.getString(3)));
            session.setIdDay(cursor.getLong(4));
            session.setDayName(cursor.getString(5));
        }

        // Closing cursor and database connection
        cursor.close();
        db.close();

        return session;
    }

    /**
     * Adds item to database
     */
    public long add(Session session)
    {
        long result;

        // Opens database in write mode
        SQLiteDatabase db = this.getWritableDatabase();

        ContentValues values = new ContentValues();
        values.put(COL_IDSPORT, session.getIdSport());
        values.put(COL_START, Utils.formatDate(session.getStart(), null));
        values.put(COL_FINISH, Utils.formatDate(session.getFinish(), null));
        values.put(COL_IDDAY, session.getIdDay());
        values.put(COL_DAYNAME, session.getDayName());
        values.put(COL_SPEED, session.getSpeed());
        values.put(COL_DIST, session.getDistance());
        values.put(COL_UNIT, session.getUnit());
        values.put(COL_CAL, session.getCalories());

        // Inserting row
        result = db.insert(TAB_SESSION, null, values);
        session.setId(result);

        // Just in case of workouts
        if (result > 0 && session.getIdSport() == Utils.SPORT_WORKOUT)
        {
            // Gettins all logged plans
            for (LogPlan log : session.getLogPlans())
            {
                log.setIdSession(session.getId());

                values = new ContentValues();
                values.put(COL_IDSESSION, log.getIdSession());
                values.put(COL_IDEXERC, log.getIdExercise());
                values.put(COL_EXERC, log.getExerciseName());
                values.put(COL_BODYPART, log.getBodypart());
                values.put(COL_POS, log.getPosition());
                values.put(COL_SETS, log.getSets());
                values.put(COL_REPS, log.getReps());
                values.put(COL_REST, log.getRest());
                values.put(COL_IDPLAN, log.getIdPlan());

                // Inserting row
                result = db.insert(TAB_LOGPLAN, null, values);
                log.setId(result);

                // If error we get out
                if (result < 1)
                {
                    break;
                }
            }
        }

        // Closing database connection
        db.close();

        return result;
    }

    /**
     * Adds item to database
     */
    public long add(Point point)
    {
        // Opens database in write mode
        SQLiteDatabase db = this.getWritableDatabase();

        ContentValues values = new ContentValues();
        values.put(COL_IDSESSION, point.getIdSession());
        values.put(COL_LONG, point.getLongitude());
        values.put(COL_LAT, point.getLatitude());
        values.put(COL_DATE, Utils.formatDate(point.getDate(), null));

        // Inserting row
        long result = db.insert(TAB_POINT, null, values);

        // Closing database connection
        db.close();

        return result;
    }

    /**
     * Adds item to database
     */
    public long add(LogPlan logPlan)
    {
        long result;

        // Opens database in write mode
        SQLiteDatabase db = this.getWritableDatabase();

        ContentValues values = new ContentValues();
        values.put(COL_IDSESSION, logPlan.getIdSession());
        values.put(COL_IDEXERC, logPlan.getIdExercise());
        values.put(COL_EXERC, logPlan.getExerciseName());
        values.put(COL_BODYPART, logPlan.getBodypart());
        values.put(COL_POS, logPlan.getPosition());
        values.put(COL_SETS, logPlan.getSets());
        values.put(COL_REPS, logPlan.getReps());
        values.put(COL_REST, logPlan.getRest());
        values.put(COL_IDPLAN, logPlan.getIdPlan());

        // Inserting row
        result = db.insert(TAB_LOGPLAN, null, values);
        logPlan.setId(result);

        // Closing database connection
        db.close();

        return result;
    }

    /**
     * Adds item to database
     */
    public long add(LogSet logSet)
    {
        long result;

        // Opens database in write mode
        SQLiteDatabase db = this.getWritableDatabase();

        ContentValues values = new ContentValues();
        values.put(COL_IDLOG, logSet.getIdLog());
        values.put(COL_DATE, Utils.formatDate(logSet.getSetDate(), null));
        values.put(COL_WEIGHT, logSet.getWeight());
        values.put(COL_UNIT, logSet.getWeightUnit());
        values.put(COL_REPS, logSet.getReps());
        values.put(COL_REST, logSet.getRest());

        // Inserting row
        result = db.insert(TAB_LOGSET, null, values);
        logSet.setId(result);

        // Closing database connection
        db.close();

        return result;
    }

    /**
     * Updates item into database
     */
    public int update(Session session)
    {
        int result = 0;

        // Opens database in write mode
        SQLiteDatabase db = this.getWritableDatabase();

        if (db == null)
        {
            return result;
        }

        ContentValues values = new ContentValues();
        values.put(COL_FINISH, Utils.formatDate(session.getFinish(), null));
        values.put(COL_DIST, session.getDistance());
        values.put(COL_SPEED, session.getSpeed());
        values.put(COL_CAL, session.getCalories());

        String[] args = new String[]{String.valueOf(session.getId())};
        String where = COL_ID + "=?";

        // Updating row
        result = db.update(TAB_SESSION, values, where, args);

        // Closing database connection
        db.close();

        return result;
    }

    /**
     * Gets logs between dates
     */
    public List<Session> getSessions(Date from, Date to)
    {
        // Opens database in read mode
        SQLiteDatabase db = this.getReadableDatabase();

        if (db == null)
        {
            return null;
        }

        String[] cols = new String[]{COL_ID, COL_IDSPORT, COL_START, COL_FINISH, COL_IDDAY, COL_DAYNAME, COL_SPEED, COL_DIST, COL_UNIT, COL_CAL};
        String where = COL_START + " BETWEEN '" + Utils.formatDate(from, null) + "' AND '" + Utils.formatDate(to, null) + "'";

        Cursor cursor = db.query(TAB_SESSION, cols, where, null, null, null, null, null);

        List<Session> sessions = new ArrayList<>();
        while (cursor.moveToNext())
        {
            Session session = new Session();
            session.setId(cursor.getLong(0));
            session.setIdSport(cursor.getLong(1));
            session.setStart(Utils.getDate(cursor.getString(2)));
            session.setFinish(Utils.getDate(cursor.getString(3)));
            session.setIdDay(cursor.getLong(4));
            session.setDayName(cursor.getString(5));
            session.setSpeed(cursor.getDouble(6));
            session.setDistance(cursor.getDouble(7));
            session.setUnit(cursor.getString(8));
            session.setCalories(cursor.getDouble(9));
            session.setContext(context);
            sessions.add(session);
        }
        // Closing cursor and database connection
        cursor.close();
        db.close();

        // return routineDayExercise
        return sessions;
    }

    /**
     * Gets logs for a day
     */
    public List<Session> getSessions(Date day)
    {
        // Opens database in read mode
        SQLiteDatabase db = this.getReadableDatabase();

        if (db == null)
        {
            return null;
        }

        String[] cols = new String[]{COL_ID, COL_IDSPORT, COL_START, COL_FINISH, COL_IDDAY, COL_DAYNAME, COL_SPEED, COL_DIST, COL_UNIT, COL_CAL};
        String where = "strftime('%Y-%m-%d', start) = ?";
        String[] args = new String[]{Utils.formatDate(day, "yyyy-MM-dd")};
        String orderBy = COL_START + " ASC ";

        Cursor cursor = db.query(TAB_SESSION, cols, where, args, null, null, orderBy, null);

        List<Session> sessions = new ArrayList<>();
        while (cursor.moveToNext())
        {
            Session session = new Session();
            session.setId(cursor.getLong(0));
            session.setIdSport(cursor.getLong(1));
            session.setStart(Utils.getDate(cursor.getString(2)));
            session.setFinish(Utils.getDate(cursor.getString(3)));
            session.setIdDay(cursor.getLong(4));
            session.setDayName(cursor.getString(5));
            session.setSpeed(cursor.getDouble(6));
            session.setDistance(cursor.getDouble(7));
            session.setUnit(cursor.getString(8));
            session.setCalories(cursor.getDouble(9));
            session.setContext(context);
            sessions.add(session);
        }

        // Closing cursor and database connection
        cursor.close();
        db.close();

        // return routineDayExercise
        return sessions;
    }

    /**
     * Gets all items from database by session
     */
    public List<Point> getAllMapPoints(long idSession)
    {
        // Opens database in read mode
        SQLiteDatabase db = this.getReadableDatabase();

        if (db == null)
        {
            return null;
        }

        String[] cols = new String[]{COL_ID, COL_LONG, COL_LAT};
        String[] args = new String[]{String.valueOf(idSession)};
        String where = COL_IDSESSION + "=?";
        String orderBy = COL_DATE + " ASC";

        Cursor cursor = db.query(TAB_POINT, cols, where, args, null, null, orderBy, null);

        List<Point> points = new ArrayList<>();

        // We ensure there is at least a record
        while (cursor.moveToNext())
        {
            Point point = new Point();
            point.setId(cursor.getLong(0));
            point.setLongitude(cursor.getDouble(1));
            point.setLatitude(cursor.getDouble(2));
            points.add(point);
        }

        // Closing cursor and database connection
        cursor.close();
        db.close();

        // return points
        return points;
    }

    /**
     * Gets all items from database by session
     */
    public List<LogPlan> getAllLogPlans(long idSession)
    {
        // Opens database in read mode
        SQLiteDatabase db = this.getReadableDatabase();

        if (db == null)
        {
            return null;
        }

        String cols = "SELECT p._id, p.id_session, p.id_exercise, p.exercise, p.bodypart, p.position, p.sets, p.reps, p.rest, count(s._id), p" + ""
                + ".id_plan ";
        String tables = "FROM log_plans p LEFT JOIN log_sets s ON (p._id = s.id_log) ";
        String where = "WHERE p.id_session = ? ";
        String groupBy = "GROUP BY p._id, p.id_session, p.id_exercise, p.exercise, p.bodypart, p.position, p.sets, p.reps, p.rest ";
        String orderBy = "ORDER BY p.position ASC ";
        String[] args = new String[]{String.valueOf(idSession)};

        Cursor cursor = db.rawQuery(cols + tables + where + groupBy + orderBy, args);

        List<LogPlan> logs = new ArrayList<>();

        while (cursor.moveToNext())
        {
            LogPlan log = new LogPlan();
            log.setId(cursor.getLong(0));
            log.setIdSession(cursor.getLong(1));
            log.setIdExercise(cursor.getLong(2));
            log.setExerciseName(cursor.getString(3));
            log.setBodypart(cursor.getString(4));
            log.setPosition(cursor.getInt(5));
            log.setSets(cursor.getInt(6));
            log.setReps(cursor.getInt(7));
            log.setRest(cursor.getInt(8));
            log.setLogged(cursor.getInt(9));
            log.setIdPlan(cursor.getLong(10));
            logs.add(log);
        }

        // Closing cursor and database connection
        cursor.close();
        db.close();

        // return routineDayExercise
        return logs;
    }

    /**
     * Gets all items from database by log plan
     */
    public List<LogSet> getAllLogSets(long idLogPlan)
    {
        // Opens database in read mode
        SQLiteDatabase db = this.getReadableDatabase();

        if (db == null)
        {
            return null;
        }

        String[] cols = new String[]{COL_ID, COL_IDLOG, COL_DATE, COL_WEIGHT, COL_UNIT, COL_REPS, COL_REST};
        String[] args = new String[]{String.valueOf(idLogPlan)};
        String where = COL_IDLOG + "=?";
        String orderBy = COL_DATE + " ASC";

        Cursor cursor = db.query(TAB_LOGSET, cols, where, args, null, null, orderBy, null);

        int i = 1;
        List<LogSet> logs = new ArrayList<>();
        while (cursor.moveToNext())
        {
            LogSet log = new LogSet();
            log.setId(cursor.getLong(0));
            log.setIdLog(cursor.getLong(1));
            log.setSetDate(Utils.getDate(cursor.getString(2)));
            log.setWeight(cursor.getDouble(3));
            log.setWeightUnit(cursor.getString(4));
            log.setReps(cursor.getInt(5));
            log.setRest(cursor.getInt(6));
            log.setPos(i);
            log.setContext(context);
            logs.add(log);
            i++;
        }

        // Closing cursor and database connection
        cursor.close();
        db.close();

        // return routineDayExercise
        return logs;
    }

    /**
     * Gets logs for a day
     */
    public List<WorkoutDay> getWorkoutLogs(long idSession)
    {
        List<WorkoutDay> logs = new ArrayList<>();

        // Retrieves log plans for session
        List<LogPlan> plans = getAllLogPlans(idSession);
        for (LogPlan plan : plans)
        {
            // Retrieves all log sets for each log plan
            for (LogSet set : getAllLogSets(plan.getId()))
            {
                WorkoutDay log = new WorkoutDay();
                log.setId(set.getId());
                log.setExercise(plan.getExerciseName());
                log.setBodypart(plan.getBodypart());
                log.setWeight(set.getWeight());
                log.setWeightUnit(set.getWeightUnit());
                log.setReps(set.getReps());
                log.setRest(set.getRest());
                log.setLogDate(set.getSetDate());
                logs.add(log);
            }
        }

        return logs;
    }

    /**
     * Closes past active sessions and returns if it has found any.
     */
    public void closeActiveWorkoutSessions()
    {
        // Opens database in write mode
        SQLiteDatabase db = this.getWritableDatabase();

        if (db == null)
        {
            return;
        }

        String table = "UPDATE sessions ";
        String cols = "SET finish = strftime('%Y-%m-%d 23:59:59', start) ";
        String where = "WHERE start < strftime('%Y-%m-%d', 'now') AND finish IS NULL ";

        // Updating row
        Cursor cursor = db.rawQuery(table + cols + where, null);
        cursor.moveToFirst();

        // Closing database connection
        cursor.close();
        db.close();
    }

    /**
     * Closes past active sessions and returns if it has found any.
     */
    public void deleteInactiveLogPlans()
    {
        // Opens database in write mode
        SQLiteDatabase db = this.getWritableDatabase();

        if (db == null)
        {
            return;
        }

        String query = "DELETE FROM log_plans WHERE _id IN (SELECT p._id FROM (SELECT p._id, p.exercise, p.id_session, count(s._id) sets FROM " +
                "log_plans p LEFT JOIN log_sets s on (p._id = s.id_log) GROUP BY p._id, p.exercise) p LEFT JOIN sessions s on (p.id_session = " +
                "s" + "._id) WHERE s.finish IS NOT NULL AND sets = 0)";

        // Deleting rows
        Cursor cursor = db.rawQuery(query, null);
        cursor.moveToFirst();

        // Closing database connection
        cursor.close();
        db.close();
    }

    /**
     * Deletes item from database
     */
    public void deleteLogPlan(long id)
    {
        // Opens database in write mode
        SQLiteDatabase db = this.getWritableDatabase();

        if (db == null)
        {
            return;
        }

        String[] args = new String[]{String.valueOf(id)};
        String where = COL_ID + "=?";

        // Deleting row
        db.delete(TAB_LOGPLAN, where, args);

        // Closing database connection
        db.close();
    }

}