/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack.utils;

import android.Manifest;
import android.content.Context;
import android.content.pm.PackageManager;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;

import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.model.CameraPosition;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Polyline;
import com.google.android.gms.maps.model.PolylineOptions;

import java.util.List;

import edu.uoc.sporttrack.R;

public class MapUtils
{
    public static final int LOCATION_PERMISSION_REQUEST_CODE = 38;

    public static final CameraPosition DEFAULT_LOCATION = new CameraPosition.Builder().target(new LatLng(41.3818, 2.1685)).zoom(17).build();

    // This tells Location Services how often to send you updates.
    private static final long UPDATE_INTERVAL = 200; // ms

    // The fastest rate for active location updates.
    public static final long FASTEST_UPDATE_INTERVAL = 200; // ms

    // The max time before batched results are delivered by location services.
    public static final long MAX_WAIT_TIME = 200;

    // Distance to avoid unnecessary updates.
    private static final float SMALLEST_DISPLACEMENT = 0.5F; //half meter

    /**
     * Checks and requests necessary app permissions on runtime.
     */
    public static void requestAllPermissions(AppCompatActivity activity)
    {
        if (ContextCompat.checkSelfPermission(activity, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED)
        {
            // Permission to access the location is missing.
            requestLocationPermission(activity, LOCATION_PERMISSION_REQUEST_CODE);
        }
    }

    /**
     * Requests the fine location permission, displaying confirmation dialog if necessary.
     *
     * @param activity  The target activity.
     * @param requestId The permission request code.
     */
    public static void requestLocationPermission(final AppCompatActivity activity, final int requestId)
    {
        final String permission = Manifest.permission.ACCESS_FINE_LOCATION;

        if (ActivityCompat.shouldShowRequestPermissionRationale(activity, permission))
        {
            // Creates method to be called when confirming permission request
            Runnable runnable = new Runnable()
            {
                @Override
                public void run()
                {
                    ActivityCompat.requestPermissions(activity, new String[]{permission}, requestId);
                }
            };

            // Shows dialog
            Utils.showDialog(activity, runnable, R.string.dialog_location_request_title, R.string.dialog_location_request_text, -1,
                             R.string.action_ok, R.string.action_cancel);
        }
        else
        {
            // Permission has not been granted yet, so it's requested and the result
            // will be received in onRequestPermissionResult().
            ActivityCompat.requestPermissions(activity, new String[]{permission}, requestId);
        }
    }

    /**
     * Checks if the result contains a result for a permission from a runtime permissions request.
     *
     * @param grantPermissions The requested permissions.
     * @param grantResults     The results from the request.
     * @param permission       The permission to be checked.
     * @return boolean
     */
    public static boolean isPermissionGranted(String[] grantPermissions, int[] grantResults, String permission)
    {
        for (int i = 0; i < grantPermissions.length; i++)
        {
            if (permission.equals(grantPermissions[i]))
            {
                return grantResults[i] == PackageManager.PERMISSION_GRANTED;
            }
        }
        return false;
    }

    /**
     * Checks app permission on runtime.
     */
    public static boolean checkPermission(Context context, String permission)
    {
        return ContextCompat.checkSelfPermission(context, permission) == PackageManager.PERMISSION_GRANTED;
    }

    /**
     * Creates the location request to start receiving updates.
     */
    public static LocationRequest createLocationRequest()
    {
        LocationRequest locationRequest = new LocationRequest();

        locationRequest.setInterval(UPDATE_INTERVAL);
        locationRequest.setFastestInterval(FASTEST_UPDATE_INTERVAL);
        locationRequest.setMaxWaitTime(MAX_WAIT_TIME);
        locationRequest.setSmallestDisplacement(SMALLEST_DISPLACEMENT);

        // Requests the most precise location possible.
        locationRequest.setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY);

        return locationRequest;
    }

    /**
     * Draws a line in a map given a list of LatLng points. Creates the polyline if none is passed.
     */
    public static Polyline drawMapLine(GoogleMap map, List<LatLng> routePoints, Polyline routeLine, int lineColor)
    {
        if (routeLine == null)
        {
            PolylineOptions options = new PolylineOptions().width(15).color(lineColor).geodesic(true);
            for (int i = 0; i < routePoints.size(); i++)
            {
                LatLng point = routePoints.get(i);
                options.add(point);
            }

            return map.addPolyline(options);
        }

        routeLine.setPoints(routePoints);

        return routeLine;
    }
}
