/*
 * Copyright (C) 2018 Pedro Perez
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uoc.sporttrack.utils;

import android.content.DialogInterface;
import android.os.Handler;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.concurrent.TimeUnit;

public class Utils
{
    public static final String ACCESS_MODE = "ACCESS_MODE";
    public static final boolean MODE_SELECTION = true;

    public static final String BUNDLE = "BUNDLE";
    public static final String BODYPART = "SELECTED_BODYPART";
    public static final String PLAN = "SELECTED_PLAN";
    public static final String EXERCISE = "SELECTED_EXERCISE";
    public static final String ROUTINE = "SELECTED_ROUTINE";
    public static final String DAY = "SELECTED_DAY";
    public static final String SPORT = "SELECTED_SPORT";
    public static final String SESSION = "SELECTED_SESSION";
    public static final String LOGPLAN = "SELECTED_LOGPLAN";
    public static final String LOGSET = "SELECTED_LOGSET";
    public static final String LOGDAY = "SELECTED_LOGDAY";

    public static final int RESULT_OK = 1;

    public static final int CODE_ADD_EXERCISE = 1;
    public static final int CODE_ADD_ROUTINE = 2;
    public static final int CODE_ADD_DAY = 3;
    public static final int CODE_SELECT_EXERCISE = 4;
    public static final int CODE_UPDATE_EXERCISE = 5;
    public static final int CODE_SELECT_ROUTINE = 6;
    public static final int CODE_UPDATE_ROUTINE = 7;
    public static final int CODE_UPDATE_DAY = 8;
    public static final int CODE_SELECT_DAY = 9;
    public static final int CODE_UPDATE_PLAN = 10;
    public static final int CODE_SELECT_SPORT = 11;
    public static final int CODE_SELECT_CURRENT = 12;
    public static final int CODE_SELECT_WORKOUT = 13;
    public static final int CODE_START_WORKOUT = 14;
    public static final int CODE_START_COUNTER = 15;
    public static final int CODE_SUMMARY = 16;
    public static final int CODE_SELECT_LOG_DAY = 17;
    public static final int CODE_SELECT_GPS = 18;
    public static final int CODE_SELECT_TIMED = 19;

    public static final int SPORT_CYCLING = 1;
    public static final int SPORT_MOUNTBIKING = 2;
    public static final int SPORT_ROWING = 3;
    public static final int SPORT_RUNNING = 4;
    public static final int SPORT_SKIING = 5;
    public static final int SPORT_TIMED = 6;
    public static final int SPORT_WORKOUT = 7;

    // Date datetimeFormatter
    private static final SimpleDateFormat datetimeFormatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.ROOT);

    // Decimal formatter
    private static final NumberFormat numberFormatter = new DecimalFormat("#.#");

    /**
     * Shows alert dialog with positive and negative buttons. Positive button needs a method to
     * be executed
     * and passed via runnable parameter. Negative button just closes de dialog.
     *
     * @param activity             The target activity.
     * @param runnable             The method to run when positive button is clicked.
     * @param titleId              The string identifier to set in dialog title.
     * @param messageId            The string identifier to set in dialog message.
     * @param iconId               The string identifier to set in dialog icon.
     * @param positiveButtonTextId The string identifier to set in dialog positive button.
     * @param negativeButtonTextId The string identifier to set in dialog negative button.
     */
    public static void showDialog(AppCompatActivity activity, final Runnable runnable, int titleId, int messageId, int iconId,
                                  int positiveButtonTextId, int negativeButtonTextId)
    {
        AlertDialog.Builder alertDialog = new AlertDialog.Builder(activity);

        alertDialog.setTitle(activity.getString(titleId));
        alertDialog.setMessage(activity.getString(messageId));

        if (iconId != -1)
        {
            alertDialog.setIcon(iconId);
        }

        if (positiveButtonTextId != -1)
        {
            // Setting positive button
            alertDialog.setPositiveButton(activity.getString(positiveButtonTextId), new DialogInterface.OnClickListener()
            {
                public void onClick(DialogInterface dialog, int which)
                {
                    new Handler().post(runnable);
                }
            });
        }

        if (negativeButtonTextId != -1)
        {
            // Setting negative button
            alertDialog.setNegativeButton(activity.getString(negativeButtonTextId), new DialogInterface.OnClickListener()
            {
                public void onClick(DialogInterface dialog, int which)
                {
                    dialog.cancel();
                }
            });
        }

        alertDialog.show();
    }

    /**
     * Parses a date in format yyyy-MM-dd HH:mm:ss and returns an object date
     *
     * @param date The date to be parsed.
     */
    public static Date getDate(String date)
    {
        if (date != null)
        {
            try
            {
                return datetimeFormatter.parse(date);
            }
            catch (ParseException e)
            {
                // Does nothing
            }
        }
        return null;
    }

    /**
     * Formats an object date to a string date in format yyyy-MM-dd HH:mm:ss
     * or uses new format passed as parameter
     *
     * @param date      The date to be parsed.
     * @param newFormat Date format to be used instead of by default.
     */
    public static String formatDate(Date date, String newFormat)
    {
        if (date != null)
        {
            if (newFormat != null)
            {
                SimpleDateFormat newFormatter = new SimpleDateFormat(newFormat, Locale.ROOT);
                return newFormatter.format(date);
            }

            return datetimeFormatter.format(date);
        }
        return null;
    }

    /**
     * Formats a decimal number in format #.#
     *
     * @param number The decimal number to be parsed.
     */
    public static String formatNumber(Double number)
    {
        if (number != null)
        {
            return numberFormatter.format(number);
        }
        return "";
    }

    /**
     * Converts from date to calendar
     *
     * @param date The date to be converted
     */
    public static Calendar toCalendar(Date date)
    {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.set(Calendar.HOUR_OF_DAY, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MILLISECOND, 0);
        return cal;
    }

    /**
     * Gets the difference between 2 dates and returns a time
     *
     * @param start  The initial date
     * @param finish The end date
     */
    public static String formatTime(Date start, Date finish)
    {
        long duration = finish.getTime() - start.getTime();
        long hours = TimeUnit.MILLISECONDS.toHours(duration);
        long mins = TimeUnit.MILLISECONDS.toMinutes(duration) - TimeUnit.HOURS.toMinutes(hours);
        long secs = TimeUnit.MILLISECONDS.toSeconds(duration) - TimeUnit.MINUTES.toSeconds(mins) - TimeUnit.HOURS.toSeconds(hours);

        return String.format("%d:%02d:%02d", hours, mins, secs);
    }

    /**
     * It converts the speed passed as m/s to km/h or mi/h depending on distanceUnit.
     * If this parameter is null or not right, it always return in km/h
     *
     * @param speedInMetresPerSecond The speed in m/s
     * @param distanceUnit           The distance unit: km or mi
     */
    public static String getFormattedSpeed(double speedInMetresPerSecond, String distanceUnit)
    {
        if (distanceUnit == null || !distanceUnit.equals("mi"))
        {
            return formatNumber(speedInMetresPerSecond * 3.6);
        }

        return formatNumber(speedInMetresPerSecond * 2.2369);
    }

    /**
     * It converts the distance passed as m to km or mi depending on distanceUnit.
     * If this parameter is null or not right, it always return in km
     *
     * @param distanceInMeters The distance in m
     * @param distanceUnit     The distance unit: km or mi
     */
    public static String getFormattedDistance(double distanceInMeters, String distanceUnit)
    {
        double distance = convertDistance(distanceInMeters, distanceUnit);
        return formatNumber(distance) + " " + distanceUnit;
    }

    /**
     * It converts the distance passed as m to km or mi depending on distanceUnit.
     * If this parameter is null or not right, it always return in km
     *
     * @param distanceInMeters The distance in m
     * @param distanceUnit     The distance unit: km or mi
     */
    public static double convertDistance(double distanceInMeters, String distanceUnit)
    {
        if (distanceUnit == null || !distanceUnit.equals("mi"))
        {
            return distanceInMeters * 0.001;
        }
        return distanceInMeters * 0.000621371;
    }

    /**
     * It returns the difference between 2 bearings
     *
     * @param bearing1 The first bearing
     * @param bearing2 The second bearing
     */
    public static double compareBearings(double bearing1, double bearing2)
    {
        return 180 - Math.abs(Math.abs(bearing1 - bearing2) - 180);
    }

}
